
package vt.cs.db;

import java.rmi.*;
import java.applet.*;
import java.util.*;

/**
 * This is a simple test applet for the database classes and interfaces.
 */
public class DatabaseTest extends Applet {

	public void init() {
		testRMI();
		testSimpleFile();
	}

	void testRMI() {
		DatabaseSet		databaseSet 	= null;

		/* Get the appropriate databaseSet */
		try {
			databaseSet = Manager.getDatabaseSet(
				"rmidb:rmi://simon/DatabaseSet/dbs");
		}
		catch (Exception e) {
			e.printStackTrace();
		}

		/* Create a new Database */
		Database myDatabase = databaseSet.newDatabase("test");
		printDatabases(databaseSet);

		/* Add a Table */
		Table	myTable = null;
		String[] cols = {"col1", "col2"};
		String[] colTypes = {"java.lang.Long", "java.lang.Long"};
		String[] colDescs = {"seconds", "meters"};
		try {
			myTable = myDatabase.newTable("a_database", 
				cols, colTypes, colDescs, "test applet");
		}
		catch(Exception e) {
			e.printStackTrace();
		}
		printDatabases(databaseSet);

		/* Add some rows to the database */
		int i;
		for (i = 0; i < 10; i++) {
			Object[] row = new Object[2];
			row[0] = new Long(i);
			row[1] = new Long(10*i);
			try {
				myTable.appendRow(row);
			}
			catch(Exception e) {
				e.printStackTrace();
			}
		}
		//printTwoColumns(myTable, 10);
		printTable(myTable);

		/* Remove the Database */
		databaseSet.deleteDatabase("test");
		printDatabases(databaseSet);

	}

	void testSimpleFile() {
		DatabaseSet databaseSet = null;
		Database	myDatabase 	= null;
		Table		myTable		= null;

		/* Get the data via http */
		String		url = "http://simon.cs.vt.edu/~isenhour/popdata.txt";
		try {
			databaseSet = Manager.getDatabaseSet("simplefile:" + url);
		}
		catch (Exception e) {
			e.printStackTrace();
		}
		printDatabases(databaseSet);

		/* 
		 * The current implementatio of SimpleFileDatabaseSet provides
		 * a single Database with a single Table, each with the
		 * URL as its name. We could also have discovered these names
		 * by calling getDatabases() and getTables().
		 */
		try {
			myDatabase = databaseSet.getDatabase(url);
			myTable = myDatabase.getTable(url);
			printAttributes(myTable);
			//printTwoColumns(myTable, 4);
			printTable(myTable);
		}
		catch (Exception e) {
			e.printStackTrace();
		}

	}

	/*
	 * A simple routine which assumes our Table has two columns of
	 * Longs.
	 */
	void printTwoColumns(Table table, int numberOfRows) {
		int i;
		for (i = 0; i < numberOfRows; i++) {
			Long elem1 = null;
			Long elem2 = null;
			try {
				elem1 = (Long)(table.get(i, 0));
				elem2 = (Long)(table.get(i, 1));
				System.out.println("Row " + i + ": " + elem1 + ", " + elem2);
			}
			catch(Exception e) {
				e.printStackTrace();
			}
		}
		System.out.println("---------------------------------------");
	}

	void printTable(Table table) {
		try {
			int		i				= 0;
			int		j				= 0;
			long	numberOfRows 	= table.rowcount();
			int		numberOfColumns	= 0;
			String	names		 	= "Column names: ";
			String	row				= "";

			for (Enumeration e = table.columnNames(); e.hasMoreElements(); ) {
				String name = (String) e.nextElement();
				names = names.concat(" " + name);
				numberOfColumns++;
			}
			System.out.println(names);
	
			for (i = 0; i < numberOfRows; i++) {
				row = new String("Row " + i + ": ");
				for (j = 0; j < numberOfColumns; j++) {
					row = row.concat("" +
						(Long)table.get(i, j) + " ");
				}
				System.out.println(row);
			}
		}
		catch(Exception e) {
			e.printStackTrace();
		}
		System.out.println("---------------------------------------");
	}

	/*
	 * Print the names of all the Databases that our DatabaseSet knows
	 * about, along with the number of Tables in each.
	 */
	void printDatabases(DatabaseSet databaseSet) {
		int 		i 		= 0;
		String[] 	dbs 	= databaseSet.getDatabases();
		Database	db 		= null;
		String[]	tables 	= null;

		System.out.println("" + dbs.length + " Databases available");
		for  (i = 0; i < dbs.length; i++) {
			System.out.println("" + i + ": " + dbs[i]);
			db = databaseSet.getDatabase(dbs[i]);
			try {
				tables = db.getTables();
			
				System.out.println("    " + tables.length + 
					" Tables in this database");
			}
			catch(Exception e) {
				e.printStackTrace();
			}
		}
		System.out.println("---------------------------------------");
	}

	/*
	 * Print all attributes associated with the Table.
	 */
	void printAttributes(Table table) {
		try {
			System.out.println("Attributes for " + table.getName());
			for (Enumeration e = table.attributeKeys(); e.hasMoreElements(); ) {
				String key = (String) e.nextElement();
				System.out.println("     " + key + " = " +
					(String) table.getAttribute(key));
			}
			System.out.println("---------------------------------------");
		}
		catch (Exception e) {
			e.printStackTrace();
		}
	}

}

