
package vt.cs.db;

import java.applet.*;

/**
 * The Manager class contains static methods for instantiating an 
 * appropriate DatabaseSet class. Eventually this will need to
 * be done by looking at applet parms, prompting the user, or inspecting
 * the contents of source data (eg, from a URL). 
 */
public class Manager {
	/**
     * A database set is instantiated based on a URL-like identification
     * string. Currently three types are supported:
     *
	 *   rmidb:[rmi url] - Connects to RMI-based database referenced in the
     *                  registry on the specified host.
     *
     *   simplefile:[url] -  Uses the simple file DatabaseSet to read a 
	 * 					table from the specified URL.
     *
     *   filedb:[url] - For large disk-based data files in the socsci project
     *                  format.
	 * 
	 * Instantiation of a DatabaseSet is performed by attempting 
	 * to instantinate an instance of vt.cs.db.[prefix].DatabaseSetImpl,
	 * and then calling the DatabaseSetImpl's init method.
	 * Eventually we may want to allow classes from other packages
	 * to be used...
	 */
	public static DatabaseSet getDatabaseSet(String identification) 
		throws ClassNotFoundException {
		String 		prefix				= null;
		int			colonidx			= 0;
		Class		databaseSetClass	= null;
		DatabaseSet	databaseSet			= null;

		colonidx = identification.indexOf(":");

		prefix = identification.substring(0, colonidx);

		databaseSetClass = Class.forName("vt.cs.db." + prefix + 
			".DatabaseSetImpl");
		try {
			databaseSet = (DatabaseSet)databaseSetClass.newInstance();
			databaseSet.init(identification.substring(colonidx + 1,
				identification.length()));
		}
		catch (Exception e) {
			e.printStackTrace();
		}
		return databaseSet;
	}

	/**
	 * I had originally envisioned this class being responsible for
	 * translation from one format to another. However, we
	 * may want to do this with DatabaseSet.addDatabase(...) method
	 * instead.
	 */
	public static void translate(String source, String newInterpreter,
		String dest) {
	}

}


