
package vt.cs.db;

import java.util.*;

/**
 * A Table of (currently) aribitrary objects. Only
 * primitive means for adding and extracting data are currently provided. 
 */
public abstract class Table extends AttributeObject {
	public static final String PREFIX = "Table";
	
	/**
	 * Returns the number of rows in the table
	 */
	public abstract long rowcount();

	/**
	 * Returns an enumeration (or array?) of column names.
	 */
	public abstract Enumeration columnNames();

	/**
	 * Returns an enumeration (or array?) of names of columns with data
	 * of the specified type. The type must be fully qualified, eg,
	 * "java.lang.Long".
	 */
	public abstract Enumeration columnNames(String type);

	/**
	 * Returns the description string associated with the specified column.
	 */
	public abstract String description(String columnName)
            throws DatabaseException;

	/**
	 * Gets the data object at the specified row and column.
	 */
	public abstract Object get(int rowNumber, int columnNumber)
        throws DatabaseException;

	/**
	 * Gets the data object at the specified row and column.
	 */
	public abstract Object get(int rowNumber, String columnName)
        throws DatabaseException;

	/**
	 * Adds a row to the table. Currently no checking for size or
	 * type consistancy is done.
	 */
	public abstract void appendRow(Object[] rowData) 
		throws DatabaseException;

	/**
	 * Temporary HACK to return an entire column as a Vector. 
     * This is being added to speed up the Graph applet for "real" use
     * of the virtual school. Similar functionality will be provided
     * by the new (and improved) interchange library.
     */
	public Vector getColumn(String columnName) throws DatabaseException {
		Vector col = new Vector();
		for (int i = 0; i < rowcount(); i++)
			col.addElement(get(i, columnName));
		return col;
    }
	public Vector getColumn(int columnNumber) throws DatabaseException {
		Vector col = new Vector();
		for (int i = 0; i < rowcount(); i++)
			col.addElement(get(i, columnNumber));
		return col;
    }

}

