/*
 * Written By:   Philip Isenhour
 * Date:         12 December 1996
 * Description:  Class for hiding details of the environment (eg, 
 *               appletviewer vs. browser vs. application) in which 
 *               the client is running.
 *
 * $Header: /home/nie/cvsroot/prototype/vt/cs/virtSchool/client/ActivityFrame.java,v 1.4 1997/02/12 20:57:31 begolej Exp $
 *
 * Copyright (c) 1996 Philip Isenhour
 * Copyright (c) 1996 Virginia Polytechnic University and State University
 */
package vt.cs.virtSchool.client;

import java.applet.*;
import java.net.*;
import java.awt.*;
import vt.cs.virtSchool.server.*;

/**
 * An <CODE>ActivityFrame</CODE> is a frame in which an activity applet
 * is run. 
 * @version $Id: ActivityFrame.java,v 1.4 1997/02/12 20:57:31 begolej Exp $
 */
public class ActivityFrame extends Frame implements Runnable, AppletStub {
  private Applet app = null;
  private Activity act = null;
  private Thread life = null;
  private ThreadGroup threadGroup = null;
  private AppletContext	context = null;
  private Dimension appSize = null;
  private TableView tableView = null;

  /**
   * @param app The activity's applet
   * @param act The activity
   * @param context The context in which we are running. Note that while we do replace the applet's AppletStub, all activity applets will use the same AppletContext.
   * @param tableView The TableView from which this activity applet was lauchned. It will be notified when the applet exits.
   */
  public ActivityFrame(Applet app, Activity act, AppletContext context,
		       TableView tableView) {
    super(act.name);

    this.app = app;
    this.act = act;
    this.tableView = tableView;
    app.setStub(this);

    MenuBar mb = new MenuBar();
    Menu m = new Menu( "Activity" );
    m.add( new MenuItem( "Restart" ) );
    m.add( new MenuItem( "Move to Back" ) );
    m.add( new MenuItem( "Close" ) );
    mb.add( m );
    setMenuBar( mb );

    setLayout( new BorderLayout() );
    add( "Center", app );
    pack();
    validate();

    threadGroup = new ThreadGroup(act.id());
    life = new Thread(threadGroup, this);
    life.start();
  }

  public void run() {
    app.init();
    validate();
    appSize = app.size();
    app.resize(act.width, act.height);
    app.start();
    validate();
    app.show();
    show();
  }

  /**
   * Currently we just look for menu events and WINDOW_DESTROY.
   */
  public boolean handleEvent( Event evt ) {
    switch ( evt.id ) {
    case Event.ACTION_EVENT:
      if ( evt.arg.equals( "Restart" ) ) {
	app.stop();
	app.destroy();
	/*
	 * Does the old thread need to be stopped?
	 */
	life = new Thread(threadGroup, this);
	life.start();
      }
      else if ( evt.arg.equals( "Move to Back" ) ) {
	toBack();
      }
      else if ( evt.arg.equals( "Close" ) ) {
	shutdown();
      }
      break;

    case Event.WINDOW_DESTROY:
      shutdown();
      //return true; // ???
      break;
    }
    return super.handleEvent(evt);
  }

  /**
   * This method attempts to gracefully shutdown the applet and 
   * dispose of the frame. It is called when the user chooses "Close"
   * from the menu or closes the applet's window. Eventually the
   * TableView should also call this on all open applets when the
   * TableView shuts down (ie, when the user leaves the room).
   *
   * All hell breaks loose if we try to destroy a (beta)
   * IFC applet, so for now we'll just hide them.
   * This won't work long-term because access to some
   * applets may have to be restricted to a single client
   * at a time, so we need to be able to actually destroy
   * the applet. This also assumes that each applet has
   * only a single window, which is not the case with,
   * for example, the Grapher applet.
   */
  void shutdown() {
    hide();
    remove(app);
    app.stop();
    life.stop();
    app.destroy();
    tableView.closeActivity(act);
    dispose();
  }

  /* ---------- AppletStub Interface Methods ---------- */

  public void appletResize(int width, int height) { 
    Dimension mySize = size();
    mySize.width += width - appSize.width;
    mySize.height += height - appSize.height;
    resize(mySize);
    appSize = app.size();
  }

  /**
   * Returns the AppletContext passed in when the ActivityFrame was
   * constructed.
   */
  public AppletContext getAppletContext() {
    return context;
  }

  public URL getCodeBase() {
    URL	codebase = null;
    try {
      codebase = new URL(Configuration.CodeBase);
    }
    catch (Exception e) {
      e.printStackTrace();
    }
    return codebase;
  }

  /**
   * Currenctly getDocumentBase returns the same thing as getCodeBase,
   * though it probably shouldn't.
   */
  public URL getDocumentBase() {
    URL	docbase = null;
    try {
      docbase = new URL(Configuration.CodeBase);
    }
    catch (Exception e) {
      e.printStackTrace();
    }
    return docbase;
  }

  /**
   * Returns the parameter as registered in the Activity's params table.
   * We DO NOT store parameter data in the system properties table like
   * a normal applet would.
   */
  public String getParameter(String name) {
    return (String)act.params.get(name);
  }

  /**
   * Currently this always returns true. Could this be what is confusing
   * IFC applets on shutdown?
   */
  public boolean isActive() {
    return true;
  }
}



