/*
 *
 * Written By:   Philip Isenhour
 * Date:         25 September 1996
 * Description:  Prototype Activity view class. 
 *
 * $Header: /home/nie/cvsroot/prototype/vt/cs/virtSchool/client/ActivityView.java,v 1.6 1997/02/19 19:43:31 begolej Exp $
 *
 * Copyright (c) 1996 Philip Isenhour
 * Copyright (c) 1996 Virginia Polytechnic University and State University
 */

package vt.cs.virtSchool.client;

import netscape.application.*;
import netscape.util.*;
import java.net.*;
import java.io.*;
import vt.cs.virtSchool.server.*;

/**
 * An ActivityView represents an available or instantiated activity
 * applet.
 * @version $Id: ActivityView.java,v 1.6 1997/02/19 19:43:31 begolej Exp $
 */
public class ActivityView extends View implements DragSource {
  public Activity act = null;
  private DragSession dragSession = null;
  private Bitmap icon = null;
  private boolean doubleClick = false;
  private String user = null;

  private int downX = 0;
  private int downY = 0;

  private ActivityFrame frame = null;

  /**
   * @param act The activity represented by this icon.
   * @param user The name of the user (client) responsible for instantiating this activity
   */
  public ActivityView(Activity act, String user) {
    super();
    this.act = act;
    this.user = user;
    icon = act.iconBitmap();
    setBounds(act.x, act.y, icon.width(), icon.height());
    //System.out.println("Instantiated new ActivityView for user: " + user);
  }

  /**
   * Replaces the view's Activity with <CODE>act</CODE> and moves
   * the view to the location specified in <CODE>act</CODE>.
   */
  public void update(Activity act) {
    this.act = act;
    setBounds(act.x, act.y, icon.width(), icon.height());
  }

  /**
   * mouseUp is interesting when we have begun linking two applets.
   * If this is called while a link is in progress we check to see
   * if the mouse is over another activity's icon. If it is we add
   * our database to their database set and vice-versa.
   */
  public void mouseUp(MouseEvent event) {
    Point			p			= null;
    View			dest		= null;
    Link			link		= null;
    LinkView		linkView	= null;
    ActivityView	destAct		= null;
		
    if (event.isMetaKeyDown()) {
      if (superview() instanceof TableView) {
	p = convertPointToView(superview(),new Point(event.x, event.y));
	dest = superview().viewForMouse(p.x, p.y);
	if (dest instanceof ActivityView) {
	  ((TableView)superview()).makeLink(this, (ActivityView)dest);
	}
	/*
	 * Tell the table to remove the light grey "temporary"
	 * link line (if it had drawn one).
	 */
	((TableView)superview()).linkFinished();
      }
    }
  }

  public void setFrame(ActivityFrame frame) {
    this.frame = frame;
  }
  public ActivityFrame getFrame() {
    return frame;
  }

  public boolean mouseDown(MouseEvent event) {
    downX = event.x;
    downY = event.y;
    if (event.isMetaKeyDown()) {
    }
    else if ((event.clickCount() == 2) && 
	     (superview() instanceof TableView)){
      doubleClick = true;
      draw();

      ((TableView)superview()).openActivity(this);

      doubleClick = false;
      draw();
    }
    return true;
  }

  /** 
   * Start a drag session. Note that the locations must be given in terms
   * of the coordinate space of the rootView.
   */
  public void mouseDragged(MouseEvent event) {
    Point p1;
    Point p2;
        
    if (event.isMetaKeyDown()) {
      if (superview() instanceof TableView) 
	((TableView) superview()).drawLink(this, event.x, event.y);
      return;
    }
    if (dragSession != null)       // Don't start more than one session!
      return;

    /* What if no icon? */
    dragSession = new DragSession(this, icon,
				  0, 0, downX, downY, "activity", null);
  }

  public void dragWasAccepted(DragSession session) {
    dragSession = null;
  }

  public void moveActivityTo(int x, int y) {
    if ((act.x != x) || (act.y != y)) {
      act.x = x;
      act.y = y;
      moveTo(x, y);
      draw();
    }
  }

  public boolean dragWasRejected(DragSession session) {
    dragSession = null;
    return true;
  }

  public View sourceView(DragSession session) {
    return this;
  }

  public void drawView(Graphics g) {
    if (icon != null)
      icon.drawCentered(g, 0, 0, width(), height());
    else {
      g.setColor(Color.blue);
      g.fillRect(0, 0, width(), height());
    }
    if (doubleClick) {
      g.setXORMode(Color.white);
      g.setColor(Color.darkGray);
      g.fillRect(0, 0, width(), height());
    }
  }

  /**
   * Returns the view's Activity's ID
   */
  public String id() {
    return act.id();
  }

}

