/*
 *
 * Written By:   Philip Isenhour
 * Date:         12 November 1996
 * Description:  View of a chat window w/input and output fields.
 *
 * $Header: /home/nie/cvsroot/prototype/vt/cs/virtSchool/world/RoomView.java,v 1
.5 1996/11/12 06:15:36 isenhour Exp $
 *
 * Copyright (c) 1996 Philip Isenhour
 * Copyright (c) 1996 Virginia Polytechnic University and State University
 */
 
package vt.cs.virtSchool.client;
 
import netscape.application.*;
import netscape.util.*;
import SharedData.*;
import vt.cs.virtSchool.server.*;

/**
 * A <CODE>ChatView</CODE> contains <CODE>TextView</CODE>s for entering
 * and viewing chat data. It is also responsible for sending and receiving
 * data on the chat channel. The interface is currently <I>very</I>
 * minimal.
 *
 * @version $Id: ChatView.java,v 1.6 1997/02/19 19:43:32 begolej Exp $
 */
public class ChatView extends View implements ChannelConsumer, TextFilter {
	private	Session		session		= null;
	private	Channel		channel		= null;
	private	Client		client		= null;
	private	TextView	outputView	= null;
	private	TextView	inputView	= null;
	private	ScrollGroup	outputGroup	= null;
	private	ScrollGroup	inputGroup	= null;
	
	public ChatView(Client client, Session session, int x, int y, 
		int w, int h) {

		super(x, y, w, h);

		/*
		 * Set ourselves as the consumer for the channel.
		 */
		this.session = session;
		this.client = client;
		try {
			channel = Channel.join(session, RoomData.CHAT_CHANNEL, client);
			channel.setConsumer(client, this);
		}
		catch (Exception e) {
			e.printStackTrace();
		}

		outputGroup = new ScrollGroup(0, 0, w, 7 * h / 10);
		outputGroup.setHasVertScrollBar(true);
		outputGroup.setBorder(new BezelBorder(BezelBorder.LOWERED, 
				Color.lightGray));
		outputGroup.scrollView().setBackgroundColor(Color.white);
		/*
		 * The following causes the bottom of the group to be fixed
		 * (i.e., it's margin, the space below it, can stretch)
		 */
		outputGroup.setVertResizeInstruction(View.BOTTOM_MARGIN_CAN_CHANGE);
		outputView = new TextView(0, 0, w - 20, 5 * h);
		outputView.setBackgroundColor(Color.white);
		outputView.setEditable(false);
		outputGroup.scrollView().setContentView(outputView);

		inputGroup = new ScrollGroup(0, 20 + 7*h/10, w, h / 5 - 10);
		inputGroup.setHasVertScrollBar(true);
		inputGroup.setBorder(new BezelBorder(BezelBorder.LOWERED, 
                Color.lightGray));
		inputGroup.scrollView().setBackgroundColor(Color.white);
		inputGroup.setVertResizeInstruction(View.BOTTOM_MARGIN_CAN_CHANGE);
		inputView = new TextView(0, 20 + 7*h/10, w - 20, h);
		inputView.setBackgroundColor(Color.white);
		inputView.setFilter(this);
		inputGroup.scrollView().setContentView(inputView);

		addSubview(outputGroup);
		addSubview(inputGroup);
		draw();
	}
	
	public void shutdown() throws SharedDataException {
		channel.leave(client);
	}

	/* ------- ChannelConsumer Interface methods ------ */
    public void dataReceived(Data data) {
		Message	msg		= null;
		Range	newText	= null;

		msg = Message.messageFromData(data);
		newText = outputView.appendString((String)msg.data);
		outputView.scrollRangeToVisible(newText);
	}
	/* ------------------------------------------------ */

	/* --------- TextFilter Interface methods --------- */
	public boolean acceptsEvent(Object textObject, KeyEvent event,
		Vector events) {

		if (event.isReturnKey()) {
			Message	chatMsg	= null;
			String	name = new String(client.getName());
			name = name.substring(0, name.indexOf("."));

			/*
			 * Send the text and clear the text field
			 */
			chatMsg = new Message(Message.CHAT_TEXT, name + ": " +
				inputView.string() + "\n");
			try {
				channel.uniformSendToAll(client, chatMsg.toData());
			}
			catch (Exception e) {
				e.printStackTrace();
			}

			inputView.setString("");
			event.key = ' ';
			//return false;
		}
		return true;
	}
	/* ------------------------------------------------ */
}


