/*
 * Written By:   Philip Isenhour
 * Date:         12 December 1996
 * Description:  Class for hiding details of the environment (eg, 
 *               appletviewer vs. browser vs. application) in which 
 *               the client is running.
 *
 * $Header: /home/nie/cvsroot/prototype/vt/cs/virtSchool/client/Environment.java,v 1.8 1997/02/19 19:43:33 begolej Exp $
 *
 * Copyright (c) 1996 Philip Isenhour
 * Copyright (c) 1996 Virginia Polytechnic University and State University
 */
package vt.cs.virtSchool.client;

import java.applet.*;
import java.net.*;
import java.awt.*;
import vt.cs.virtSchool.server.*;

/**
 * The <CODE>Environment</CODE> class was originally intended to simply
 * hide the details of the "environment" in which the client was running
 * (eg, appletviewer vs. browser vs. application). Specifically it was
 * to be responsible for spawning activity applets in the appropriate
 * way (launching appletviewer or calling <CODE>showDocument</CODE>).
 * <P>Recent work, however, suggests that a different approach may be
 * better: Implementing a custom <CODE>AppletStub</CODE> which effectively
 * provides the same functionality as launching another appletviewer
 * but without the security and performance implications.  The (private)
 * <CODE>ActivityFrame</CODE> class does this.
 * <P>Among other advantages, this approach will allow much better
 * control over activity applet processes than would otherwise be possible.
 * Specifically, it should be possible to always know which clients
 * are actually viewing a specific applet.
 * 
 * @version $Id: Environment.java,v 1.8 1997/02/19 19:43:33 begolej Exp $
 */
public class Environment {
	private static AppletContext	context	= null;
	private static Applet			applet	= null;

	public static void init(Applet app) {
		applet = app;
		context = applet.getAppletContext();
	}

	/**
	 * Currently all of the interesting work is done when <CODE>execute</CODE>
	 * instantiates an ActivityFrame, so this should probably be moved
	 * to the ActivityFrame class (and the Environment class eliminated
	 * entirely).
	 */
	public static ActivityFrame execute(Activity act, TableView tableView) {
		String url = act.url;
		try {
/*
			if (context == null) {
				String command[] = new String[2];
				command[0] = Configuration.AppletViewer;
				command[1] = url;
				Process proc = Runtime.getRuntime().exec(command);
			}
			else {
				context.showDocument(new URL(url));
			}
*/
			Applet newApp = (Applet)Class.forName(
				Configuration.LabAppletsPackage +
				"." + act.appletName + "." + act.appletName).newInstance();
			return new ActivityFrame(newApp, act, context, tableView);
		}
		catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

	public static java.awt.Image getImage(URL url) {
		return context.getImage(url);
	}

	public static netscape.application.Bitmap getBitmap(String urlStr) {
		try {
         return netscape.application.AWTCompatibility.bitmapForAWTImage( getImage(
               new java.net.URL(vt.cs.virtSchool.server.Configuration.CodeBase + "images/" + urlStr)
                            )
                        );
		}
		catch (Exception e) {e.printStackTrace();}
		return null;
	}
}

