/*
 *
 * Written By:   Philip Isenhour
 * Date:         1 November 1996
 * Description:  Prototype non-como-based LabRoom replacement.
 *
 * $Header: /home/nie/cvsroot/prototype/vt/cs/virtSchool/client/RoomView.java,v 1.8 1997/05/15 17:31:22 begolej Exp $
 *
 * Copyright (c) 1996 Philip Isenhour
 * Copyright (c) 1996 Virginia Polytechnic University and State University
 */
 
package vt.cs.virtSchool.client;
 
import netscape.application.*;
import netscape.util.*;
import SharedData.*;
import mlsoft.mvt.*;
import vt.cs.virtSchool.server.*;
import vt.cs.db.*;

/**
 * The <CODE>RoomView</CODE> is responsible for creating all of the views
 * associated with a room: the TableView, ShelfView, ChatView, UserList,
 * and a TextView for an HTML-formatted description of the room.
 *
 * @version $Id: RoomView.java,v 1.8 1997/05/15 17:31:22 begolej Exp $
 */
public class RoomView extends View implements Client, WindowOwner {
  private MlTabView tabView = null;
  private ShelfView shelf = null;
  private TableView table = null;
  private UserList users = null;
  private TextView description = null;
  private ChatView chat = null;
  private RoomData room = null;
  private Session roomSession = null;
  private Channel shelfChannel = null;
  private Channel tableChannel = null;

  private String name = null;

  private WindowOwner otherOwner = null;

  /**
   * @param width Width of the view
   * @param height Height of the view
   * @param room Reference to RoomData object on the server
   * @param name Name of the user for whom this view is being instantiated
   */
  public RoomView(int width, int height, RoomData room, String name) {
    super();
    setBuffered(true);

    sizeTo(width, height);

    this.room = room;

    try {
      this.name  = room.getUniqueClientName(name);
      roomSession = SessionFactory.createSession(room.getSessionURL());
			
      roomSession.join(this);
      shelfChannel = Channel.join(roomSession, RoomData.SHELF_CHANNEL, 
				  this);
      tableChannel = Channel.join(roomSession, RoomData.TABLE_CHANNEL, 
				  this);
    }
    catch (Exception e) {
      e.printStackTrace();
    }

    tabView = new MlTabView(width()/2 + 10, 10, width()/2 - 30,
			    height()/2 - 30);
    tabView.setValue("tabsPerRow", 2);

    shelf = new ShelfView(this, shelfChannel, 0, 0, width()/2 - 30, 
			  height()/2 - 30);
    tabView.add("Activities", shelf);

    users = new UserList(0, 0, 20,  50);
    ScrollGroup userScroll = 
      new ScrollGroup(0, 0, 20,  50);

    userScroll.scrollView().setContentView(users);
    userScroll.setHasVertScrollBar(true);
    userScroll.setHasHorizScrollBar(true);
    userScroll.setBorder(new LineBorder(Color.black));

    tabView.add("Users", userScroll);

    ScrollGroup descriptionScroll =
      new ScrollGroup(0, 0, 20,  50);
    descriptionScroll.setHasVertScrollBar(true);
    descriptionScroll.setHasHorizScrollBar(true);
    description = new TextView(0, 0, descriptionScroll.scrollView().width()
- descriptionScroll.vertScrollBar().width() + 250, 300);
    descriptionScroll.scrollView().setBackgroundColor(Color.white);
    description.setBackgroundColor(Color.white);
    description.setEditable(false);

    try {
      description.importHTMLFromURLString(room.getDescriptionURL());

      description.setVertResizeInstruction(View.HEIGHT_CAN_CHANGE);
      descriptionScroll.scrollView().setContentView(description);
      descriptionScroll.setBorder(new LineBorder(Color.black));
      tabView.add("Description", descriptionScroll);
	
      table = new TableView(this, tableChannel, room,
			    10, 10, width()/2 - 20, height() - 50);

      addSubview(tabView);
      addSubview(table);

      createActivityViews(room.getShelfActivities(), shelf);
      createActivityViews(room.getTableActivities(), table);
    }
    catch (Exception e) {
      e.printStackTrace();
    }

    chat = new ChatView(this, roomSession, width()/2 + 10, height()/2,
			width()/2 - 30, height()/2 - 40);

    addSubview(chat);

    users.update(roomSession);
  }

  /**
   * Adds all of the activities in <CODE>acts</CODE> to the ActivityContainer
   * <CODE>dest</CODE>. Used to initialize the ShelfView and TableView.
   */
  void createActivityViews(java.util.Hashtable acts, ActivityContainer dest) {
    Object				obj		= null;
    java.util.Vector	links	= new java.util.Vector();

    /*
     * All of the activities need to be added first, since links can
     * only be created between existing activities.
     */
    for (java.util.Enumeration e = acts.elements(); e.hasMoreElements();) {
      obj =  e.nextElement();
      if (obj instanceof Activity)
	dest.activityAdded((Activity)obj);
      else
	links.addElement(obj);
    }
    for (java.util.Enumeration e = links.elements(); e.hasMoreElements();) {
      dest.linkAdded((Link)e.nextElement());
    }
  }

  /* ----------- Client Interface methods ----------- */
  public String authenticate(String challenge) {
    debug("RoomView challenged = "+challenge);
    return null;
  }
  public String getName() {
    debug("getName - "+name);
    return name;
  }
  public void clientChanged(ClientEvent event) {
    debug("RoomView clientChanged = "+event);
  }

  /* ------------------------------------------------ */

  /**
   * This is a blatent, ugly hack.  We currently need for the window
   * this view is in to have two "owners" -- ourselves and the list of
   * rooms. This view needs to know when the window closes so that
   * it can leave the room's session and channels. The list of rooms
   * needs to know so that it will know to create a new window the
   * next time the user clicks on the room's name in the list.
   */
  public void setOtherOwner(WindowOwner other) {
    otherOwner = other;
  }

  /* ------- WindowOwner Interface methods --------- */
  public boolean windowWillShow(Window aWindow) {
    if (otherOwner != null)
      otherOwner.windowWillShow(aWindow);
    return true;
  }
  public void windowDidShow(Window aWindow) { }
  public boolean windowWillHide(Window aWindow) {
    /*
     * Present a dialog box here?
     */
    if (otherOwner != null)
      otherOwner.windowWillHide(aWindow);
    return true;
  }
  public void windowDidHide(Window aWindow) { 
    try {
      roomSession.deleteObserver(users);
      chat.shutdown();
      /*
       * Need to call shutdown() methods in TableView and ShelfView
       * also. (Need to write them, first.) It would probably be
       * good if the TableView and ShelfView were also responsible
       * for joining and leaving the appropriate channels.
       */
      tableChannel.leave(this);
      shelfChannel.leave(this);

      roomSession.leave(this);
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }
  public void windowDidBecomeMain(Window aWindow) { }
  public void windowDidResignMain(Window aWindow) { }
  public void windowWillSizeBy(Window aWindow, Size deltaSize) { }
  /* ------------------------------------------------ */

  private static  final boolean debug = false;
  private void debug(String msg) {
    if (debug)
      System.out.println(getClass().getName()+": "+msg);
  }
 
}

