/*
 *
 * Written By:   Philip Isenhour
 * Date:         25 September 1996
 * Description:  Prototype Shelf view class. The shelf contains available
 *				 (rather than instantiated) activities.
 *
 * $Header: /home/nie/cvsroot/prototype/vt/cs/virtSchool/client/ShelfView.java,v 1.4 1997/02/19 19:43:37 begolej Exp $
 *
 * Copyright (c) 1996 Philip Isenhour
 * Copyright (c) 1996 Virginia Polytechnic University and State University
 */

package vt.cs.virtSchool.client;

import netscape.application.*;
import netscape.util.*;
import vt.cs.virtSchool.server.*;
import SharedData.*;

/**
 * A ShelfView contains icons for available activities which may be
 * dragged onto the table for use.
 *
 * @version $Id: ShelfView.java,v 1.4 1997/02/19 19:43:37 begolej Exp $
 */
public class ShelfView extends View implements DragDestination,
	ActivityContainer {
	private	Hashtable	activities 	= null;
	private	Client		client		= null;
	private	Channel		channel		= null;
	private	ActivityChannelConsumer	consumer	= null;

	public ShelfView(Client client, Channel channel, int x, int y, 
		int width, int height) {
		super(x, y, width, height);
		this.client = client;
		this.channel = channel;
		this.consumer = new ActivityChannelConsumer(client, channel, this);
		activities = new Hashtable();

		setHorizResizeInstruction(View.WIDTH_CAN_CHANGE);
	}

	public void drawView(Graphics g) {
		g.setColor(Color.white);
		g.fillRect(0, 0, width(), height());
		g.setColor(Color.black);
		g.drawRect(0, 0, width(), height());
	}

	public DragDestination acceptsDrag(DragSession dragSession,
		int x, int y) {
		/* Check type first? */
		return this;
	}

	public boolean dragDropped(DragSession dragSession) {
		ActivityView	av 			= null;
		Rect			dragBounds 	= null;
		Point			p 			= null;

		if (dragSession.source() instanceof ActivityView) {
			av = (ActivityView) dragSession.source();
			/*
			 * For now, it doesn't make sense to drag new things onto
			 * the shelf. Eventually we may want to allow dragging from
			 * some sort of a library view.
			 */
			if (av.descendsFrom(this)) {
				dragBounds = dragSession.destinationBounds();
				av.moveActivityTo(dragBounds.x, dragBounds.y);
				draw();
				return true;
			}
		}
		return false;
	}

	public boolean dragEntered(DragSession dragSession) {
		return true;
	}
          
	public boolean dragMoved(DragSession dragSession) {
		return true;
	}
          
	public void dragExited(DragSession dragSession) {
	}

	/**
	 * When a subview (activity icon) is moved we send a message to the
	 * other client's so that the icon moves on thier shelves also.
	 */
	public void subviewDidMove(View aSubview) {
		if (aSubview instanceof ActivityView) {
			/*
			 * Send a message to the other clients
			 */
			try {
				channel.uniformSendToOthers(client, new Data(Message.toByteArray(
					new Message(Message.ACTIVITY_MOVE, 
					((ActivityView)aSubview).act))));
			}
			catch(Exception e) {
				e.printStackTrace();
			}
		}
	}

	/* ---------- ActivityContainer Interface Methods ---------- */
	public void activityAdded(Activity act){
		ActivityView actView	= null;
		actView = new ActivityView(act, client.getName());
        activities.put(act.id(), actView);
		addSubview(actView);
    }
    public void activityMoved(Activity act){
		ActivityView actView    = null;
		actView = (ActivityView) activities.get(act.id());
		if (actView != null) {
			actView.update(act);
			draw();
		}
    }

    public void activityRemoved(Activity act){}

	/*
	 * Throw an exception on these?
	 */
    public void activityClaimed(Activity act){}
    public void activityReleased(Activity act){}
    public void linkAdded(Link link){}
    public void linkRemoved(Link link){}

}


