/*
 *
 * Written By:	James Begole
 * Date:			2 April 1996
 * Description:	Histogram Charting Tool
 * 
 */

 
/*
 * $Header: /home/nie/cvsroot/prototype/vt/cs/virtSchool/labapplets/Histogram/HistoChart.java,v 1.3 1996/12/17 04:49:58 isenhour Exp $
 *
 * Copyright (c) 1996 James "Bo" Begole
 * Copyright (c) 1996 Virginia Polytechnic University and State University
 */

package vt.cs.virtSchool.labapplets.Histogram;

import vt.cs.virtSchool.util.*;

import java.rmi.RemoteException;

import java.awt.*;
import java.util.*;

import vt.cs.db.*;
import vt.cs.db.widgets.ifc.*;

public class HistoChart extends Canvas implements DatabaseConsumer {
  /**
      +-------------------------------------+
      |          Title                      |
      +-------------------------------------|
      |left |left |                         |
      |label|Ret- |                         |
      |     |icule|        Chart Area       |
      |     |     |                         |
      |     |     |                         |
      |     |     |                         |
      |     |     |                         |
      |     |     |                         |
      |-----+-----+-------------------------+
      |   XXX     |    bottom Reticule      |
      |-----------+-------------------------|
      |          bottom label               |
      +-------------------------------------+
*/

  private final boolean debug = false;
  private Rectangle chartArea = null;
  private Rectangle title = null;
  private Rectangle leftReticule = null;
  private Rectangle leftLabel = null;
  private Rectangle bottomReticule = null;
  private Rectangle bottomLabel = null;

  private int tickLength = 5;
  private int reticuleWidth = 2*tickLength;	
  private int maxPointSize = 20;
  private int minPointSize = 2;
	
  private int pointDiameter = 2;
	
  //DataSets dataSets = null;
  Table		table  = null;
  String	column = null;
  boolean	dataChanged = true;

  Vector binnedPoints = null;
  Vector linePoints = null;
  int maxBinSize = 1;
  Dimension border = new Dimension(10, 10);

  public HistoChart() {
    if (debug) System.out.println("HistoChart: ");
    //this.databaseSet = databaseSet;
  }

  public void reBuild() {
    binnedPoints = getBins();
    repaint();
  }
	
  public void paint(Graphics g) {
    if (debug) System.out.println("paint: ");
    update(g);
  }

  Dimension offDimension;
  Image offImage;
  Graphics offGraphics;
	
  public void update(Graphics g) {
    if (debug) System.out.println("update: ");
		
    Dimension d = size();
    Rectangle r = bounds();

    if (debug) System.out.println("d= "+d);
    if (debug) System.out.println("r= "+r);
		
    if ( (offGraphics == null) || (d.width != offDimension.width) || (d.height != offDimension.height) ) {
      offDimension = d;
      offImage = createImage(d.width, d.height);
      offGraphics = offImage.getGraphics();
      offGraphics.setFont(g.getFont());
      title = new Rectangle(0, 0, d.width-1, 3*offGraphics.getFontMetrics().getHeight());
      leftLabel = new Rectangle(0, title.height+1, 3*offGraphics.getFontMetrics().getHeight(), d.height-(title.height+3*offGraphics.getFontMetrics().getHeight()));  
      leftReticule = new Rectangle(leftLabel.width+1, leftLabel.y, reticuleWidth, leftLabel.height);
      chartArea = new Rectangle(leftReticule.x+leftReticule.width+1, leftLabel.y,
				0, leftLabel.height);
      chartArea.width = d.width-chartArea.x-1;
      bottomReticule = new Rectangle(chartArea.x, chartArea.y+chartArea.height, chartArea.width, reticuleWidth);
      bottomLabel = new Rectangle(0, bottomReticule.y+bottomReticule.height+1, d.width-1, 0);
      bottomLabel.height = d.height-bottomLabel.y-1;
    }
		
    offGraphics.setColor(Color.white);
    offGraphics.fillRect(0, 0, offDimension.width, offDimension.height);

    if (dataChanged)
      binnedPoints = getBins();

    drawBins(offGraphics.create(chartArea.x, chartArea.y, chartArea.width, chartArea.height ));
		
    drawBottomReticule(offGraphics.create(bottomReticule.x, bottomReticule.y, bottomReticule.width, bottomReticule.height));
    drawLeftReticule(offGraphics.create(leftReticule.x, leftReticule.y, leftReticule.width, leftReticule.height));
    drawBottomLabel(offGraphics.create(bottomLabel.x, bottomLabel.y, bottomLabel.width, bottomLabel.height ));
    drawLeftLabel(offGraphics.create(leftLabel.x, leftLabel.y, leftLabel.width, leftLabel.height ));
    drawTitle(offGraphics.create(title.x, title.y, title.width, title.height ));
		
    //		drawRects(offGraphics);
    g.drawImage(offImage, 0, 0, null);
  }
	
  private void drawBins(Graphics g) {
    if (debug) System.out.println("drawBins: ");

    Rectangle area = g.getClipRect();
    //		g.setColor(Color.lightGray);
    g.fillRect(0, 0, area.width, area.height);

	if (binnedPoints.size() <= 0)
		return;

    int binWidth = (area.width- 2*border.width) / binnedPoints.size();

    linePoints = new Vector();
    linePoints.addElement(new Point(0, area.height));
    int j = 0;
    for(Enumeration e = binnedPoints.elements(); e.hasMoreElements(); j++) {
      Vector bin = (Vector)e.nextElement();
      int finalY = drawBin(g.create(border.width + j*binWidth , 0, binWidth,offDimension.height), bin);
      linePoints.addElement(new Point((border.width + j*binWidth +binWidth/2), finalY));
    }
    linePoints.addElement(new Point(area.width-1, area.height));

    g.setColor(Color.blue);
    Point prev = null;
    boolean startDraw = false;
    for(Enumeration e = linePoints.elements(); e.hasMoreElements(); j++) {
      Point thisPoint = (Point)e.nextElement();
			
      if (startDraw) {
	g.drawLine(prev.x, prev.y, thisPoint.x, thisPoint.y);
      } else {
	startDraw = true;
      }
      prev = thisPoint;
    }
		
  }
	
  private int drawBin(Graphics g, Vector bin) {
    Rectangle area = g.getClipRect();
    if (debug) System.out.println("area="+area);
    int lastY = area.height;
		
    g.setColor(Color.red);

    int binHeight = area.height - 2*border.height;

    int binElementHeight = binHeight/maxBinSize;

    pointDiameter = Math.min(((area.width*2)/3), maxPointSize);
    pointDiameter = Math.min(pointDiameter, (int)Math.ceil((binElementHeight*2)/3));
    pointDiameter = Math.max(pointDiameter, minPointSize);
    if (debug) System.out.println(area);
    if (debug) System.out.println("pointDiameter = "+pointDiameter+" binElementHeight = "+binElementHeight
				  +" area.height = "+area.height+" maxBinSize = "+maxBinSize);
		
    int j = 0;
    for(Enumeration e = bin.elements(); e.hasMoreElements(); j++) {
      Number dataPoint = (Number)e.nextElement();
			
      g.fillOval(
		 (area.width-pointDiameter)/2, 
		 area.height-(pointDiameter+j*(pointDiameter +pointDiameter/2)), 
		 pointDiameter, 
		 pointDiameter);
      lastY = area.height-(pointDiameter+j*(pointDiameter +pointDiameter/2));
			
      if (debug) System.out.println("Element: "+dataPoint+" drawn at y= "+(area.height-(border.height+20+j*30)));
    }
    return lastY;
  }

	
  private void drawBottomReticule(Graphics g) {
    Rectangle area = g.getClipRect();
    if (debug) System.out.println("drawBottomReticule "+area);
		
    g.setColor(Color.black);
		
    g.drawLine(0, 0, area.width, 0);

	if (binnedPoints.size() <= 0)
		return;

    int numTicks =  binnedPoints.size();
    int tickDistance = area.width/(numTicks);
		
    for(int i = 0; i< numTicks; i++) {
      g.drawLine(i*tickDistance, 0, i*tickDistance, tickLength);
    }
    // Draw the last tick
    g.drawLine(area.width, 0, area.width, tickLength);

  }

  private void drawLeftReticule(Graphics g) {
    Rectangle area = g.getClipRect();

    g.setColor(Color.black);
		
    g.drawLine(area.width-1, 0, area.width-1, area.height);
    int tickDistance = pointDiameter+pointDiameter/2;
    int numTicks = area.height/tickDistance;
		
    for(int i = 0; i<numTicks; i++) {
      g.drawLine(area.width-tickLength-1, area.height-(i*tickDistance+1), area.width, area.height-(i*tickDistance+1));
    }
    // Draw the last tick
    g.drawLine(area.width-tickLength-1, 0, area.width, 0);
  }
	
  private void drawBottomLabel(Graphics g) {
    Rectangle area = g.getClipRect();
		
    g.setColor(Color.black);
    int numTicks =  binnedPoints.size();
	if (numTicks <= 0)
		return;
    int tickDistance = bottomReticule.width/numTicks;
		
    for(int i = 0; i< numTicks; i++) {
      g.drawString("Bin: "+i, bottomReticule.x+i*tickDistance, g.getFontMetrics().getHeight()+2);
    }
  }
	
  private void drawTitle(Graphics g) {
	String titleString = null;
    if (debug) System.out.println("drawTitle: ");
    Rectangle area = g.getClipRect();
		
    g.setColor(Color.black);
		
    Font oldFont = g.getFont();
		
    g.setFont(new Font(oldFont.getName(), Font.BOLD, oldFont.getSize()+4));
    //String titleString = new String("Histogram of Reaction Times");
	if (column != null)
	    titleString = new String(column);
	else
	    titleString = new String("");
    int stringWidth = g.getFontMetrics().stringWidth(titleString);
    int stringHeight = g.getFontMetrics().getHeight();

    // Center it
    int yPos = ((area.height-stringHeight)/2 )+ stringHeight/2;
    g.drawString(titleString, (area.width-stringWidth)/2,  yPos);
		
    g.setFont(oldFont);
  }
	
  private void drawLeftLabel(Graphics g) {
  }
	
  // for debugging
  public void drawRects(Graphics g) {
    g.setColor(Color.yellow);
    g.drawRect(chartArea.x, chartArea.y, chartArea.width, chartArea.height);
    g.setColor(Color.black);
    g.drawRect(title.x, title.y, title.width, title.height);
    g.setColor(Color.red);
    g.drawRect(leftReticule.x, leftReticule.y, leftReticule.width, leftReticule.height);
    g.setColor(Color.blue);
    g.drawRect(leftLabel.x, leftLabel.y, leftLabel.width, leftLabel.height);
    g.setColor(Color.green);
    g.drawRect(bottomReticule.x, bottomReticule.y, bottomReticule.width, bottomReticule.height);
    g.setColor(Color.darkGray);
    g.drawRect(bottomLabel.x, bottomLabel.y, bottomLabel.width, bottomLabel.height);
  }


  private static final int NUMBINS = 5;
  private static final int BINDIVISOR = 10;

  private Vector getBins() {
    long minPointValue = Long.MAX_VALUE;
    long maxPointValue = Long.MIN_VALUE;

	long	numPoints	= 0;
	long	i			= 0;
	Number	current		= null;

    Vector allPoints = new Vector();
    Vector tempBins = new Vector();

	if ((table != null) && (column != null)) {
		try {
			numPoints = table.rowcount();
			for(i = 0; i < numPoints; i++) {
				current = (Number)table.get((int)i, column);
				allPoints.addElement(current);
				if (current.longValue() < minPointValue)
					minPointValue = current.longValue();
				if (current.longValue() > maxPointValue)
					maxPointValue = current.longValue();
			}
		}
		catch (Exception e) {
			e.printStackTrace();
		}

    long allRange = maxPointValue - minPointValue;
    int numBins = Math.max(NUMBINS, allPoints.size()/BINDIVISOR);
    long binRange = (allRange/numBins)+1;
    if (debug) System.out.println("numBins= "+numBins+" allRange= " +allRange+" binRange= "+binRange);

    // create the bins of vectors
    for (i=0; i<numBins; i++) {
      tempBins.addElement(new Vector());
    }

    int binIndex = 0;	
    Vector tempVector = null;
    for(Enumeration e = allPoints.elements(); e.hasMoreElements(); ) {
      Number dataPoint = (Number)e.nextElement();
			
      binIndex = (int)((dataPoint.longValue()-minPointValue)/binRange);
      if (debug) System.out.println("binIndex= "+binIndex+" dataPoint= "+dataPoint);
      tempVector = (Vector)tempBins.elementAt(binIndex);
      tempVector.addElement(dataPoint);
    }

    maxBinSize = 1;	
    for(Enumeration e = tempBins.elements(); e.hasMoreElements(); ) {
      tempVector = (Vector)e.nextElement();
      if (tempVector.size() > maxBinSize) {
	maxBinSize = tempVector.size();
      }
    }		

		dataChanged = false;
	}
    return tempBins;
  }

    /* ---------- DatabaseConsumer interface methods ---------- */
    public void databaseSetSelected(DatabaseSet dbSet, boolean doubleClicked) {
    }
    public void databaseSelected(Database db, boolean doubleClicked) {
    }
    public void tableSelected(Table table, boolean doubleClicked) {
    }
    public void columnSelected(Table table, String columnName,
        boolean doubleClicked) {
		try {
			Object sample = table.get(0, columnName);
			if (sample instanceof Number) {
        		//System.out.println("Selected Column: " + columnName);
				this.table = table;
				this.column = columnName;
				dataChanged = true;
				repaint();
			}
			else {
				System.out.println("Non-numeric column selected");
			}
		}
		catch (Exception e) {
			e.printStackTrace();
		}
    }
    /* -------------------------------------------------------- */

}


