/*
 *
 * Written By: Philip Isenhour
 * Date:       21 August 1996
 * Descriptions: Simple tool for dumping out raw data from datasets
 *
 */

/* 
 * $Header: /home/nie/cvsroot/prototype/vt/cs/virtSchool/labapplets/RawData/RawData.java,v 1.2 1996/09/17 19:48:05 isenhour Exp $
 * Copyright (c) 1996 Philip Isenhour
 * Copyright (c) 1996 Virginia Polytechnic University and State University
 */

package vt.cs.virtSchool.labapplets.RawData;

import vt.cs.virtSchool.util.*;
import vt.cs.virtSchool.objectManager.ObjectManager;
import vt.cs.widgets.MultiColumn;
 
import java.rmi.Remote;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.rmi.Naming;
 
import java.applet.*;
import java.awt.*;
import java.net.*;
import java.util.*;

public class RawData extends Applet implements Runnable {

	private Thread			thd;
	private ObjectManager 	objectManager = null;
	private DataSets		dataSets = null;
	private TextField		title;
	private MultiColumn		table;
	private TextArea		log;

	public void init() {
		String cols[] = {"Index", "Value"};

		objectManager = RemoteObject.getObjectManager(this);
		dataSets = RemoteObject.getDataSets(this, objectManager);

		setLayout(new BorderLayout());
		title = new TextField(getParameter("DataSets"));
		title.setEditable(false);
		table = new MultiColumn(cols, 5);
		log = new TextArea();
		log.setEditable(false);
		add("North", title);
		add("Center", table);
		add("South", log);
		thd = new Thread(this);
		thd.start();
	}

	public void run() {
		int 	iterations = 0;
		int 	lastCount = -1;
		int 	lastLogCount = -1;
		Vector 	keys = null;
		while (true) {
			try {
				Vector	points = dataSets.getPoints();
				int 	count = points.size();
				int 	i = 0;
				int		selected = -1;

				String 	logString = "";
				if (lastLogCount == -1) {
					keys = dataSets.getKeys("java.lang.String");
					if ((keys != null) && (keys.size() > 0))
						logString = "Using String data with key: " + 
							(String)keys.firstElement();
				}

				if ((keys != null) && (keys.size() > 0)) {
					Vector	logEntries = 
						dataSets.get((String)keys.firstElement());
					int		logCount = logEntries.size();
					if (logCount != lastLogCount) {
						for (i = lastLogCount; i < logCount - 1; i++) {
							DataObject entry = (DataObject)
								logEntries.elementAt(i+1);
							logString = logString + "\n" +
								(String)entry.value() + " [" + 
								entry.time() + "]";
						}
						log.appendText(logString);
						lastLogCount = logEntries.size() - 1;
					}
				}
				else
					log.setText("");

				if (count != lastCount) {
					title.setText(getParameter("DataSets") + " (" +
						Integer.toString(count) + " elements) -- " +
						Integer.toString(iterations++));

					selected = table.selected();
					table.clear();
					for (Enumeration pts = points.elements(); 
						pts.hasMoreElements();) {
						Long 	dataPt = (Long)pts.nextElement();
						String	entry[] = new String[2];
						entry[0] = Integer.toString(i++);
						entry[1] = dataPt.toString();
						table.addItem(entry);
					}
					table.select(selected);
					lastCount = count;
				}
			} 
			catch (RemoteException e) {
				System.out.println("RawData.run(): " + e);
			}
			try {
				thd.sleep(2000);
			}
			catch (InterruptedException e) {}
		}
	}

	public boolean handleEvent(Event e) {
		if (e.id == Event.WINDOW_DESTROY)
			System.exit(0);
		return false;
	}
}
 
