
//
//	Rectangle.java
//
//	Copyright 1996,	Black Diamond Software.	 All rights reserved.
//
//	Written	by: Jason Beaver and Tyler Gingrich
//

package	vt.cs.virtSchool.labapplets.WhiteBoard;

import netscape.application.*;
import netscape.util.*;

public class Rectangle extends Graphic {
	Rect rect;
	Point origin;
	Size size;

	// Anchor points...
	private	static final int UPPER_LEFT = 0;
	private	static final int LOWER_LEFT = 1;
	private	static final int UPPER_RIGHT = 2;
	private	static final int LOWER_RIGHT = 3;
	private	static final int UPPER = 4;
	private	static final int LOWER = 5;
	private	static final int RIGHT = 6;
	private	static final int LEFT =	7;

	public Rectangle() {
		super();
	}

	public Rectangle(Rect aRect) {
		super();

		rect = aRect;
	}

	// Called to set the receiver up for creating...
	public void createGraphic(MouseEvent event,SimpleDrawView view)	{
		super.createGraphic(event,view);

		rect = new Rect(event.x,event.y,0,0);
		anchor = UPPER_LEFT;
	}

	// This	is called repeatedly as	the user manipulates the graphic...
	public void updateGraphic(MouseEvent event,SimpleDrawView view)	{
		super.updateGraphic(event,view);

		if (view.currentOperation() == SimpleDrawView.CREATING ||
			view.currentOperation()	== SimpleDrawView.RESIZING) {
			switch (anchor)	{
				case UPPER_LEFT:
					rect.width = event.x-rect.x;
					rect.height = event.y-rect.y;
					break;
				case LOWER_LEFT:
					rect.width = event.x-rect.x;
					rect.height = rect.height-event.y+rect.y;
					rect.y = event.y;
					break;
				case UPPER_RIGHT:
					rect.width = rect.width-event.x+rect.x;
					rect.height = event.y-rect.y;
					rect.x = event.x;
					break;
				case LOWER_RIGHT:
					rect.width = rect.width-event.x+rect.x;
					rect.height = rect.height-event.y+rect.y;
					rect.x = event.x;
					rect.y = event.y;
					break;
				case UPPER:
					rect.height = event.y-rect.y;
					break;
				case LOWER:
					rect.height = rect.height-event.y+rect.y;
					rect.y = event.y;
					break;
				case RIGHT:
					rect.width = rect.width-event.x+rect.x;
					rect.x = event.x;
					break;
				case LEFT:
					rect.width = event.x-rect.x;
					break;
			}
		}
		view.graphicChanged(this);
	}

	// Called when the user	finishes manipulating the graphic...
	public void finishManipulation(MouseEvent event,SimpleDrawView view) {
		// Reset rect so width and height are positive...
		rect.x = (rect.x+rect.width<rect.x)?(rect.x+rect.width):rect.x;
		rect.y = (rect.y+rect.height<rect.y)?(rect.y+rect.height):rect.y;
		rect.width = Math.abs(rect.width);
		rect.height = Math.abs(rect.height);
	}

	// This	is called whenever the graphic should display itself...
	public void drawGraphic(Graphics g) {
		int minX, minY;

		super.drawGraphic(g);

		minX = (rect.x+rect.width<rect.x)?(rect.x+rect.width):rect.x;
		minY = (rect.y+rect.height<rect.y)?(rect.y+rect.height):rect.y;

		if (isFilled())	{
			g.setColor(fillColor());
			g.fillRect(minX,minY,Math.abs(rect.width),Math.abs(rect.height));
		}
		g.setColor(strokeColor());
		g.drawRect(minX,minY,Math.abs(rect.width),Math.abs(rect.height));
	}


	// Returns a Vector of the graphic's handles (as Points)...
	public Vector handles()	{
		int minX, minY;
		int width, height;
		Point lowerLeft, lowerRight, upperLeft,	upperRight;
		Point upper, lower, right, left;

		Vector handles = new Vector();

		minX = (rect.x+rect.width<rect.x)?(rect.x+rect.width):rect.x;
		minY = (rect.y+rect.height<rect.y)?(rect.y+rect.height):rect.y;

		width =	Math.abs(rect.width);
		height = Math.abs(rect.height);

		upperLeft = new	Point(minX,minY);
		handles.addElement(upperLeft);

		upper =	new Point((minX	+ width/2),minY);
		handles.addElement(upper);

		upperRight = new Point(minX+width,minY);
		handles.addElement(upperRight);

		left = new Point(minX,(minY+height/2));
		handles.addElement(left);

		right =	new Point((minX+width),(minY+height/2));
		handles.addElement(right);

		lowerLeft = new	Point(minX,(minY+height));
		handles.addElement(lowerLeft);

		lower =	new Point((minX+width/2),(minY+height));
		handles.addElement(lower);

		lowerRight = new Point((minX+width),(minY+height));
		handles.addElement(lowerRight);

		return handles;
	}

	// Returns the anchor corresponding to the hit handle...
	public int anchorForHitHandle(MouseEvent event,SimpleDrawView view) {
		Vector handles = handles();

		for (int i=0,count=handles.count();i<count;i++)	{
			Point handlePoint = ((Point)handles.elementAt(i));

			if (hitHandle(event,handlePoint)) {
				switch (i) {
					case 0:	return LOWER_RIGHT;
					case 1:	return LOWER;
					case 2:	return LOWER_LEFT;
					case 3:	return RIGHT;
					case 4:	return LEFT;
					case 5:	return UPPER_RIGHT;
					case 6:	return UPPER;
					case 7:	return UPPER_LEFT;
				}
			}
		}
		return super.anchorForHitHandle(event,view);;
	}

	// Used	for hit	detection.
	public boolean intersectsRect(Rect aRect) {
		Rect comparisonRect = new Rect(rect);

		if(comparisonRect.width	<1) comparisonRect.width = 1;
		if(comparisonRect.height <1) comparisonRect.height = 1;

		return comparisonRect.intersects(aRect);
	}

	public void moveBy(int deltaX, int deltaY) {
		rect.moveBy(deltaX,deltaY);
	}
}
