/*
 *
 * Written By:   Philip Isenhour
 * Date:         October 1996
 * Description:  Server application class for virtschool.
 *
 * $Header: /home/nie/cvsroot/prototype/vt/cs/virtSchool/client/RoomView.java,v
1.4 1997/01/07 20:52:58 isenhour Exp $
 *
 * Copyright (c) 1996 Philip Isenhour
 * Copyright (c) 1996 Virginia Polytechnic University and State University
 */

package vt.cs.virtSchool.server;

import SharedData.*;
import java.net.*;
import java.util.*;
import vt.cs.db.rmidb.Server;
import vt.cs.db.rmidb.ServerImpl;
import vt.cs.db.*;

/**
 * Server application class.
 *
 * @version $Id Exp $
 */
public class WorldServer {
  public static final int 	PORT = 6789;
  public static final String 	NAME = "WorldServer";
  public static final String sessionType = "socket";


  static	Session		serverSession 	= null;
  static	Session		clientSession 	= null;
  static	Server		serverStub		= null;
  static	Thread		serverThread	= null;

  static	Table		rooms			= null;
  static	Table		activities 		= null;

  /*
	static	Registry	jsdaRegistry	= null;
	static	java.rmi.registry.Registry	rmiRegistry	= null;
*/

  public synchronized static void main(String args[]) {
    String     	url			= null;
    String     	hostname    = null;
    String		portstring  = null;
    int        	hostport    = 0;
    Properties	properties	= null;


    serverThread = Thread.currentThread();
    System.setSecurityManager(new java.rmi.RMISecurityManager());
		
    startServers();

    hostname = getArg(args, "server");
    if (hostname == null) {
      try {
	hostname = InetAddress.getLocalHost().getHostName();
      }
      catch(Exception e) {
	e.printStackTrace();
      }
    }

    portstring = getArg(args, "port");
    if (portstring == null) 
      hostport = PORT;
    else 
      hostport = Integer.parseInt(portstring);

    /*
     * Not sure if we'll need this, but for now, start one "WorldServer"
     * session. Room sessions will be created when RoomData objects
     * are instantiated. 
     */
    url = "coll://" + hostname + ":" + hostport + 
      "/" + WorldServer.sessionType+ "/Session/" + NAME;

    try {
      serverSession = SessionFactory.createSession(url);
      //      serverSession = new SharedData.socket.socketSession(NAME);
      //      Naming.rebind(url, serverSession);

      // Get a client session on that session url
      clientSession = SessionFactory.createSession(url);

      DatabaseSet dbSet = 
	Manager.getDatabaseSet(Configuration.ConfigDatabaseURL);
      Database db = dbSet.getDatabase("Virtual School Configuration");

      activities = db.getTable("Activities");
      rooms = db.getTable("Rooms");

      for(int i = 0; i < rooms.rowcount(); i++) {
	RoomDataImpl room =
	  new RoomDataImpl( (String)rooms.get(i, "RoomName")
			    + "." + RoomData.ROOMTAG,
			    (String)rooms.get(i, "RoomName"), 
			    (String)rooms.get(i, "DescriptionURL"), 
			    hostname, hostport);

	StringTokenizer acts = new StringTokenizer(
						   (String) rooms.get(i, "Activities"), ",");

	Activity act = null;

	while (acts.hasMoreTokens()) {
	  String activityName = acts.nextToken();
	  act = null;
	  /*
	   * Yikes! Just need to do this once somewhere....
	   */
	  for (int j = 0; j < activities.rowcount() && (act == null);
	       j++) {
	    if (activityName.equals((String)activities.get(j,
							   "AppletName"))) {
	      //  int type = Activity.READS_DATA;
	      int type = Activity.NO_IO;
	      if (((Boolean)activities.get(j, "ReadsData")).booleanValue())
		type = Activity.READS_DATA;

	      if (((Boolean)activities.get(j, 
					   "ProducesData")).booleanValue()) {
		if (type == Activity.READS_DATA) {
		  type = Activity.READS_AND_WRITES_DATA;
		} else {
		  type = Activity.WRITES_DATA;
		}
	      }
					
	      room.addShelfActivity( new Activity(
		  ((Long)activities.get(j, "XPosition")).intValue(),
		  ((Long)activities.get(j, "YPosition")).intValue(),
		  ((Long)activities.get(j, "Width")).intValue(),
		  ((Long)activities.get(j, "Height")).intValue(),
		  ((String)activities.get(j, "AppletName")),
		  Configuration.CodeBase +
		  ((String)activities.get(j, "IconURL")), "",
		  ((String)activities.get(j, "FullName")),
		  type));
	    }
	  }
	}
      }
    }
    catch (Exception e) {
      e.printStackTrace();
    }
    System.out.println("WorldServer started...");
  }

  private static String getArg(String args[], String argName) {
    int length = args.length;
    for (int i = 0; i < length; i++) {
      if (args[i].equals("-" + argName)) {
	if (++i < length) {
	  return(args[i]);
	}
      }
    }
    return null;
  }

  /**
   * Returns an array of refs to all available rooms. If called before
   * the server has finished initializing all rooms, it will block.
   */
  public synchronized static RoomData[] getRooms(String hostname) {
    int			i				= 0;
    String		url				= null;
    RoomData[]	roomDatas		= null;
    boolean		serverStarted	= false;

    try {
      if (rooms == null) {
	DatabaseSet dbSet = 
	  Manager.getDatabaseSet(Configuration.ConfigDatabaseURL);
	Database db = dbSet.getDatabase(
					"Virtual School Configuration");

	activities = db.getTable("Activities");
	rooms = db.getTable("Rooms");
      }

      while (!serverStarted) {
	try {
	  java.rmi.Naming.lookup("rmi://" +
				 hostname + "/" + RoomData.ROOMTAG + "/" +
				 (String)rooms.get((int)rooms.rowcount() - 1, 
						   "RoomName")
				 + "." + RoomData.ROOMTAG);
	  serverStarted = true;
	}
	catch(java.rmi.NotBoundException e) {
	  /* Ignore these */
	  System.out.println("Waiting for server to come up.");
	  Thread.sleep(1000);
	}
      }

      roomDatas = new RoomData[(int)rooms.rowcount()];
      for(i = 0; i < rooms.rowcount(); i++) {
	roomDatas[i] = (RoomData) java.rmi.Naming.lookup("rmi://" +
			     hostname + "/" + RoomData.ROOMTAG + "/" +
			     (String)rooms.get(i, "RoomName") + 
			     "." + RoomData.ROOMTAG);
      }
    }
    catch (Exception e) {
      e.printStackTrace();
    }
    return roomDatas;
  }

  /**
   * Currently startServers will just start up a vt.cs.db.rmidb.Server
   * object if one isn't already running. Eventually we may want to start RMI
   * and SharedData registries here as well.
   */
  static void startServers() {
    ServerImpl	dbServer	= null;

    /*
      try {
      rmiRegistry = java.rmi.registry.LocateRegistry.getRegistry(
      Configuration.RMIRegistryPort);
      }
      catch (java.rmi.ConnectException e) {
      try {
      rmiRegistry = java.rmi.registry.LocateRegistry.createRegistry(
      Configuration.RMIRegistryPort);
      }
      catch (Exception ex) {
      ex.printStackTrace();
      System.exit(1);
      }
      }
      catch (Exception e) {
      e.printStackTrace();
      System.exit(1);
      }
      */

    try {
      serverStub = (Server) java.rmi.Naming.lookup(Server.NAME);
    }
    catch (Exception e) {
      try {
	dbServer = new ServerImpl(Server.NAME);
	java.rmi.Naming.bind(Server.NAME, dbServer);
	serverStub = (Server) java.rmi.Naming.lookup(Server.NAME);
      }
      catch (Exception ex) {
	ex.printStackTrace();
	System.exit(1);
      }
    }
  }

  public static void shutdown() {
    if (serverSession != null) {
    }
    if (clientSession != null) {
    }
  }

}


