/*
 *
 * Written By:   Philip Isenhour
 * Date:         17 September 1996
 * Description:  A DataObject is the encapsulation of an arbitrary data
 *				 value produced by a producer and consumed by the consumer.
 *
 * Usage Notes:
 *
 */
 
package vt.cs.virtSchool.util;
 
import java.util.*;

/**
 * A DataObject is the encapsulation of an arbitrary data value produced by 
 * a producer and consumed by the consumer. Instances of this class are
 * constructed by ProducerStreams from raw data values provided by the
 * producer. A ConsumerStream can then return either individual DataObjects
 * or vectors of data objects depending on the type of query made. The
 * consumer applet can then extract the original object provided by the
 * producer, as well as timestamp and other information.
 */
public class DataObject {
	Object 	value = null;
	long	time = 0;
	/*
     * Will eventually store ref to somewhere that we can get label, unit,
     * and source info.
	 */

	/**
	 * Constructs a new DataObject with the given value and time.
	 */
	public DataObject(Object value, long time) {
		this.value = value;
		this.time = time;
	}

	/**
	 * Returns the raw data value provided by the producer.
 	 */
	public Object value() {
		return this.value;
	}

	/**
	 * Returns the timestamp associated with this data point (in millisecs?)
	 */
	public long time() {
		return this.time;
	}

	/**
	 * Returns the name of this data value's class
	 */
	public String className() {
		return this.value.getClass().getName();
	}
}




