/*
 *
 * Written By:   James "Bo" Begole and Craig Struble
 * Date:         31 May 1996
 * Description:  World Client.  Main Student interface to Virtual School
 *
 * Usage Notes:
 *
 * This is hacked from ComoIRCCLient.java
 * 
 */
 
/*
 * $Header: /home/nie/cvsroot/prototype/vt/cs/virtSchool/worldClient/WorldClient.java,v 1.1.1.1 1996/09/17 05:23:17 isenhour Exp $
 * Modified for use by the NIE project.
 * Copyright (c) 1996 James "Bo" Begole, Craig Struble
 * Copyright (c) 1996 Virginia Polytechnic University and State University
 */


package vt.cs.virtSchool.worldClient;

import java.awt.*;
import java.applet.Applet;
import java.net.*;
import java.io.*;
import java.util.*;

import como.awt.*;
import como.commlet.*;
import como.util.*;
import como.io.*;
import como.sys.*;

//import virtSchool.util.*;

public class WorldClient extends Applet implements ComoIRCFrontEnd, SessionAccessor {
  static final boolean debug = false;        // Do we want debug information
  static int IRC_PORT = 6667;
  static final String VERSION = "V0.2";      // Some kind of version

  /**
   * What name do all of the IRC channels start with so we can recognize
   * which ones we own.
   */
  public static final String WORLD_NAME = "nieworld";

  /**
   * The actual IRC prefix used
   */
  public static final String WORLD_CHANNEL_PREFIX = "#"+WORLD_NAME+"~";

  /**
   * The delimiter between session names. Used to separate session levels.
   */
  public static final String SESSION_DELIMITER = "$";

  /**
   * The name for the level 0 (world) sessions
   */
  public static final String SESSION_LEVEL_0 = WORLD_NAME;

  /**
   * The name for the level 1 (topic) sessions
   */
  public static final String SESSION_LEVEL_1 = "room";

  /**
   * The name for the level 2 (group) sessions
   */
  public static final String SESSION_LEVEL_2 = "table";

  /**
   * The name for the level 3 (intragroup) sessions
   */
  public static final String SESSION_LEVEL_3 = "application";

  /**
   * The delimeter for the level 0 session within the IRC channel name
   */
  public static final String LEVEL_0_DELIMITER = SESSION_DELIMITER+SESSION_LEVEL_0+SESSION_DELIMITER ; 
  
  /**
   * The delimeter for the level 1 session within the IRC channel name
   */
  public static final String LEVEL_1_DELIMITER = SESSION_DELIMITER+SESSION_LEVEL_1+SESSION_DELIMITER ; 

  /**
   * The delimeter for the level 2 session within the IRC channel name
   */
  public static final String LEVEL_2_DELIMITER = SESSION_DELIMITER+SESSION_LEVEL_2+SESSION_DELIMITER ; 

  /**
   * The delimeter for the level 3 session within the IRC channel name
   */
  public static final String LEVEL_3_DELIMITER = SESSION_DELIMITER+SESSION_LEVEL_3+SESSION_DELIMITER ; 

	
  int ircport; // variable for the IRC port being used

  /*
   * Parameters for the applet
   */
  String[][] params = {
    {"IRCPort","int","IRC Server port to connect to"},
    {"HelpPage","http-address","A http-address of the help-page"}
  };

  /**
   * Initialize the world client. This reads the IRCPortparameter, the
   * HelpPage parameter and sets up the applet.
   */
  public void init()	{
    URL helppage;
    String s;

    //
    // Get the IRC port to connect to
    //
    s = getParameter("IRCPort"); 
    try { 
      ircport = (new Integer(s.trim())).intValue(); 
    } 
    catch (Exception e) {
      ircport = IRC_PORT; 
    }

    //
    // Get the help page URL
    //
    s = getParameter( "HelpPage" ); 
    if( s == null || !s.startsWith( "http://" ) )
      s = "http://www4.informatik.uni-erlangen.de/IMMD-IV/Projects/como/doc/como-irc-help.html";

    try {
      helppage = new URL( s );
    } catch( MalformedURLException e ) {
      helppage = null;
      Debug.msg( 9, "ComoIRCClient.init(): no helppage" );
    }

    /////////////////

    setupApplet();
  }

  /**
   * Start the applet. Currently does nothing.
   */
  public void start() {
    return;
  }
  
  /**
   * Stop the applet. Currently does nothing.
   */
  public void stop() {	
    return;
  }

  /**
   * Returns information about the world client.
   */
  public String getAppletInfo() {
    return "NIE Virtual School World Client "+VERSION+" (c) James Begole and Craig Struble";
  }

  /**
   * Returns information about parameters used by the world client.
   */
  public String[][] getParameterInfo() {
    return params;
  }

  static int COMMLET_LINES = 7;
  static int USER_LINES = 8;
  static int CHANNEL_LINES = 6;
  static int IRC_MAX_NICK_LENGTH = 9;
  static int IMGSIZE = 96;

  static final int COMMLET_NAME = 0;
  static final int COMMLET_ICON_NAME = 1;
  static final int COMMLET_DESCRIPTION = 2;

  static final String USER_CHOICE_ALL = "All Available Users";
  static final String USER_CHOICE_USER = "Users matching pattern";
  static final String USER_CHOICE_CHANNEL = "Users on channel";

  static int EVENT_INVITATION_ACCEPTED = 15015;
  static int EVENT_INVITATION_REJECTED = 15016;

  List userlist = null;
  User[] users = null;
  Button userupdate = null;
  Choice userchoice = null;
  TextField userpattern = null;
  Checkbox usercomoonly = null;
	
  Button commletstartbutton = null;
  Panel commletpanel = null;
  TextField commletdescriptionfield = null;
  ImageButton commletimagebuttons[] = null;
  String commletnames[][] = null;
	
  List channellist = null;
  IrcChan[] channels = null;
  Hashtable channelHT = null;
  Button channelupdate = null;
  TextField channelpattern = null;
  Checkbox channelcomoonly = null;
  Button channeljoinbutton = null;
  Button channelviewbutton = null;
  Button channelinvitebutton = null;

  Button miquit = null;
  Button sessionsButton = null;
  SessionsDialog sessionsDialog = null;
  SessionsDialog testTableDialog = null;

  Panel userpanel = null;
  Hashtable userHT = null;
  Panel channelpanel = null;
  Panel roomsPanel = null;
  SessionsPanel sessionsPanel = null;
  UsersPanel usersPanel = null;

  TextField logTF = null;	
  TextField nickTF = null;
  TextField nameTF = null;
  User ego = null;
	
  ServerIRC server = null;

  Image map = null;

  /**
   * Sets up the world client by creating the initial menu, getting the map
   * of the world, and opening the login dialog.
   */
  public void setupApplet() {

    try {
      map = getImage(new URL(getDocumentBase(),"worldmap"));
    }
    catch (Exception e) {
      map = null;
    }

    setLayout(new BorderLayout());
    makeMenu();

    noConnection();	// starts the login process
  }
	
  private String whoWhereString = "Who is Logged In";
  Panel menubar = new Panel();

  /**
   * Creates buttons at the top of the world client for various operations.
   */
  public void makeMenu() {
    Panel menfile = new Panel();
    Button menhelp = new Button("Help");

    miquit = new Button("Quit");

    menfile.add( miquit );

    menubar.add( menfile );
    menubar.add( menhelp );
		
    add("North", menubar);
  }
	
  /**
   * Displays the list boxes in the opening frame
   */
  private void showLists() {
    if (WorldClient.debug) {
      // Debug makes the user list appear. Not tested well.
      System.out.println("WorldClient: showLists: user");
      createUserPanel();
    } else {
      updateuserlist();
    }

    if (WorldClient.debug) {
      // Debug makes the channel list appear. Not tested well.
      System.out.println("WorldClient: showLists: channel");
      createChannelPanel(); 
    } else {
      updatechannellist();
    }

    if (WorldClient.debug) {
      System.out.println("WorldClient: showLists: rooms");
    }
    createRoomsPanel();

    //
    // Change the layout and make sure all packed windows are no longer
    // packed so we can use the new layout.
    //
    setLayout(new VertLayout(VertLayout.STRETCH));
    removeAll();

    add(menubar);
    if (userpanel != null)
      add(userpanel);

    if (channelpanel != null)
      add(channelpanel);

    add(roomsPanel);
    

    //
    // Status information panel
    //
    logTF = new TextField();
    logTF.setEditable(false);
    Panel p = new Panel();
    p.setLayout(new BorderLayout());
    p.add("West",new Label("Status:"));
    p.add("Center",logTF);
    add(p);

    layout();
    paintAll(getGraphics());
  }
	
  /**
   * This creates the panel that displays the available rooms in the world
   */
  private void createRoomsPanel() {
    if (WorldClient.debug)
      System.out.println("WorldClient: createRoomsPanel: roomsPanel");
    usersPanel = new UsersPanel(this, "People Logged Into Session:", 10);
    sessionsPanel = new SessionsPanel(this, "Rooms", LEVEL_1_DELIMITER, 
				   usersPanel, 10);
    roomsPanel = new Panel();

    // Use gridbag to lay things out in 75%, 25% proportions
    GridBagLayout gbl = new GridBagLayout();
    GridBagConstraints gbc = new GridBagConstraints();

    roomsPanel.setLayout(gbl);

    gbc.fill = GridBagConstraints.BOTH;
    gbc.weightx = 0.75;
    gbl.setConstraints(sessionsPanel,gbc);
    roomsPanel.add(sessionsPanel);

    gbc.weightx = 0.25;
    gbl.setConstraints(usersPanel,gbc);
    roomsPanel.add(usersPanel);
  }

  /**
   * This creates the panel that shows the users on the IRC server
   */
  private void createUserPanel() {
    System.out.println("WorldClient: createUserPanel:");
    userpanel = new Panel();
    userpanel.setLayout(new BorderLayout());
    userlist = new List(USER_LINES,true);
    userpanel.add("Center",userlist);
    Panel p = new Panel();
    p.setLayout(new BorderLayout());
    userchoice=new Choice();
    userchoice.addItem(USER_CHOICE_ALL);
    userchoice.addItem(USER_CHOICE_USER);
    userchoice.addItem(USER_CHOICE_CHANNEL);
    p.add("West",userchoice);
    userpattern = new TextField("*");
    p.add("Center",userpattern);
    usercomoonly = new Checkbox("Como Users Only");
    usercomoonly.setState(true);
    //		p.add(usercomoonly);
    userupdate = new Button("Update");
    p.add("East",userupdate);
    userpanel.add("North",p);
    updateuserlist();
  }

  /**
   * This creates the panel that shows the raw channel information
   */
  private void createChannelPanel() {
    channelpanel = new Panel();
    channelpanel.setLayout(new BorderLayout());
    channellist = new List(CHANNEL_LINES,false);
    channelpanel.add("Center",channellist);
    Panel p = new Panel();
    p.setLayout(new BorderLayout());
    p.add("West",new Label("Available channels matching "));
    channelpattern = new TextField("*",10);
    p.add("Center",channelpattern);
    channelcomoonly = new Checkbox("Como channels Only");
    channelcomoonly.setState(true);
    //		p.add(channelcomoonly);
    channelupdate = new Button("Update");
    p.add("East",channelupdate);
    channelpanel.add("North",p);
    channeljoinbutton = new Button ("Join this channel");
    channeljoinbutton.disable();
    channelviewbutton = new Button ("See who is on this channel");
    channelviewbutton.disable();
    channelinvitebutton = new Button("Invite selected Users");
    channelinvitebutton.disable();
    Panel q = new Panel();
    q.add(channelviewbutton);
    q.add(channeljoinbutton);
    q.add(channelinvitebutton);
    channelpanel.add("South",q);
    updatechannellist();
  }

  /**
   * This updates the list of channels by querying the IRC server
   */
  private synchronized void updatechannellist() {
    showStatus("Updating Channel List...");

    //
    // Get the channels from the server. If the channel list is being shown,
    // then we can use the filter field to filter out which channels we want
    // to list.
    //
    if (channellist != null) {
      channellist.delItems(0,channellist.countItems()-1);
      channels = server.getChannels(channelpattern.getText(),channelcomoonly.getState());
    } else {
      channels = server.getChannels("*", false);
    }

    
    channelHT = new Hashtable();
    String s = null;

    //
    // Add the channels to the channel list
    //
    for (int i = 0;i<channels.length;i++) {
      IrcChan c = channels[i];
      s = c.commletname + " session with topic '" + c.topic+"'";
      if (channellist != null)
	channellist.addItem(s);
      channelHT.put(s,new Integer(i));
    }

    if (channellist != null) {
      channelinvitebutton.disable();
      channeljoinbutton.disable();
      channelviewbutton.disable();
    }
    clearStatus();
  }
	
  /**
   * Update the list of users by querying the IRC server
   */
  private synchronized void updateuserlist() {
    System.out.println("WorldClient: updateuserlist:");
    showStatus("Updating User List...");
    String pattern = "*";
    boolean comoonly = false;

    userHT = new Hashtable();
    
    //
    // Set the pattern for filtering the user list if the userlist panel
    // exists
    //
    if (userlist != null) {
      userlist.delItems(0,userlist.countItems()-1);
      pattern = userpattern.getText().trim();
      comoonly = usercomoonly.getState();
      if (userchoice.getSelectedItem() == USER_CHOICE_ALL)
	pattern = "*";
    }

    //
    // Get the list of users
    //
    users = server.getUsers(pattern,comoonly);

    //
    // List the users
    //
    for (int i = 0;i<users.length;i++)
      {
	User u = users[i];
	StringBuffer s = new StringBuffer();

	if (u.containsKey(u.NAME)) s.append(u.get(u.NAME).toString());

	if (u.containsKey(u.COMMENT)) s.append(" ("+u.get(u.COMMENT).toString()+")"); 
	if (userlist != null) 
	  userlist.addItem(s.toString());
	userHT.put(s.toString(),new Integer(i));
      }

    clearStatus();
    if (commletstartbutton != null) commletstartbutton.disable();
  }

  /**
   * Clear the status field
   */
  private void clearStatus() {
    if (ego != null)
      showStatus(ego.get(User.NAME).toString() + "'s Como Session");
    else
      showStatus("");
  }

  /**
   * Create a panel that launches commlets. Not used now.
   */
  private void createCommletPanel() {
    Vector lines;

    Panel commletbuttonpanel = new Panel();
    commletpanel = new Panel();

    commletpanel.setLayout(new BorderLayout());
    commletbuttonpanel.setLayout( new FlowLayout() );

    try {
      URL url = new URL(getDocumentBase(),server.PATH_COMMLET_LIST);
      lines = StreamLine.loadLines( new DataInputStream(url.openStream()) );
    } catch( Exception e ) {
      commletpanel.add("North",new Label("Sorry, could not load available commlets!"));
      commletpanel.add("South",new Label("Please try again later."));
      return;
    }

    int nrlines = lines.size();
    commletnames = new String[nrlines][3];  // CommletName, PicName, Description
    commletimagebuttons = new ImageButton[nrlines]; // ImageButtons for them

    for( int i = 0; i < nrlines; i++ ) {
      String line = (String)lines.elementAt(i);
      int first = line.indexOf( ':' );
      int second = line.indexOf( ':', first+1 );

      commletnames[i][COMMLET_NAME] = line.substring( 0, first-1 ).trim();
      commletnames[i][COMMLET_ICON_NAME] = line.substring( first+2, second-1 ).trim();
      commletnames[i][COMMLET_DESCRIPTION] = line.substring( second+2 ).trim();

      try {
	URL url = new URL(getDocumentBase(), server.PATH_COMMLET_ICONS+commletnames[i][1] );
	Image image = getImage( url );
	commletimagebuttons[i] = new ImageButton( image, image, IMGSIZE, IMGSIZE );
      }
      catch( Exception e ) {
	commletimagebuttons[i] = new ImageButton( IMGSIZE, IMGSIZE, commletnames[i][0] );
      }

      commletbuttonpanel.add( commletimagebuttons[i] );
    }

    commletdescriptionfield = new TextField();
    commletdescriptionfield.setEditable( false );

    commletpanel.add("North",new Label("Available Commlets"));
    commletpanel.add("Center", commletbuttonpanel );
    commletpanel.add("South", commletdescriptionfield );
  }

  /**
   * Logout from the IRC server
   */
  public void logout() {
    // Ask the user if he wants to log out?
    if( server != null )
      server.logout();
    hide();
  }

  /**
   * Handle the action event
   * @param evt The raw event.
   * @param what The object that had an action
   */
  public boolean action(Event evt,Object what) {
	
    if (whoWhereString.equals(what)) {		// display who's logged in and where
      if (sessionsDialog == null) {
	Component comp = this;

	while (!(comp instanceof Frame)) {
	  comp = comp.getParent();
	}
	Frame tempFrame=(Frame)comp;
	sessionsDialog = new SessionsDialog(tempFrame, "Rooms Available", LEVEL_1_DELIMITER, this);
      }
      sessionsDialog.show();
    }
    else if ("Test Mars Room".equals(what)) {
      if (testTableDialog == null) {
	Component comp;
	comp = getParent();
	while (!(comp instanceof Frame)) {
	  comp = comp.getParent();
	}
	Frame tempFrame=(Frame)comp;
	testTableDialog = new SessionsDialog(tempFrame, "Tables Available", LEVEL_1_DELIMITER+"Mars$table$", this);
      }
      testTableDialog.show();
    } else if (evt.target == miquit) {
      logout();
      return true;
    }
    else if (ego != null) {
      if (evt.target == userlist) {
	// double clicked on a user -> show info about that user
	// (ask server)
	showStatus("Retrieving Information about "+evt.arg.toString());
	//	User u = server.getUser(users[((Integer)userHT.get(evt.arg)).intValue()]);
	User u = users[((Integer)userHT.get(evt.arg)).intValue()];
				
	Frame f = new SmartFrame();
	f.setLayout(new VertLayout(VertLayout.STRETCH));
	f.add(new UserPanel(u,""));
	f.pack();
	f.setTitle( "User Information" );
	f.show();
	clearStatus();
	return true;
      }

      if ((evt.target == userupdate) || (evt.target==userpattern) || (evt.target == userchoice))
	{
	  updateuserlist();
	}

      if ((evt.target == channelupdate)|| (evt.target==channelpattern))
	{
	  updatechannellist();
	}

      if ((evt.target == channellist) || (evt.target == channelviewbutton)) 
	{
	  IrcChan chan = null;
	  if (evt.target == channelviewbutton) {
	    if (channellist.getSelectedIndex() != -1) 
	      chan = channels[channellist.getSelectedIndex()];
	  }
	  else
	    if (evt.arg != null) chan = channels[((Integer)channelHT.get(evt.arg)).intValue()];
	  String s = "";	
	  if (chan != null) {
	    s = "#"+chan.channel+"~"+chan.commletname;
	    showStatus("Updating User List to display users on channel "+s);
	    userpattern.setText(s);
	    userchoice.select(2);
	    updateuserlist();
	  }
	  else {
	    //MODAL
	  }
	}

      if (evt.target == channeljoinbutton)
	{
	  showStatus("Joining Channel "+channellist.getSelectedItem());
	  server.joinChannel(channels[channellist.getSelectedIndex()]);
	}

      if (evt.target == channelinvitebutton)
	{
	  int[] selints = userlist.getSelectedIndexes();
	  showStatus("Inviting "+selints.length+" users to channel "+channellist.getSelectedItem());
	  for (int i=0;i<selints.length;i++) 
	    server.sendInvitation(channels[channellist.getSelectedIndex()],users[selints[i]]);
	}
    }
    return false;		
  }

  /**
   * Handle an invitation to communication
   * @param u
   * @param chan
   */
  public void handleInvitation(User u,IrcChan chan) {
    if (WorldClient.debug)
      System.out.println("Received Invitation");
    SmartFrame f = new SmartFrame(this,
				  "Yes, accept",new Event(this,EVENT_INVITATION_ACCEPTED,chan),
				  "No, reject",new Event(this,EVENT_INVITATION_REJECTED,null));
    f.setTitle("Invitation received...");
    Panel p = new Panel();
    p.setLayout(new VertLayout(VertLayout.CENTER));
    String name = u.get(u.NAME).toString();
    String comment = u.get(u.COMMENT).toString();
    if (WorldClient.debug)
      System.out.println("Received Invitation from "+name);
    if( comment.length() > 0 )
      p.add(new Label("Invitation from "+name+" ("+comment+")"));
    else
      p.add(new Label("Invitation from "+name));
    p.add(new Label("Would you like to join a "+chan.commletname+ " Session?"));
    p.add(new Label("Topic : "+chan.topic));
    f.add("North",p);
    f.pack();
    f.show();
  }		

  /**
   * Handles an AWT event
   * @param evt
   */
  public boolean handleEvent(Event evt) {

    if (evt.id == Event.MOUSE_ENTER )
      {
	if( commletnames == null ) return false;

	// look if the mouse moved over a commlet-imagebutton 
	for( int i = 0; i < commletnames.length; i++ )
	  {
	    if (evt.target == commletimagebuttons[i]) {
	      commletdescriptionfield.setText( commletnames[i][COMMLET_NAME]+" : "+
					       commletnames[i][COMMLET_DESCRIPTION] );

	      return true;
	    }
	  }

	return false;
      }

    if (evt.id == Event.WINDOW_DESTROY ) {
      logout();
      return true;
    }

    if (evt.id == EVENT_INVITATION_REJECTED) {
      ((Frame)evt.target).dispose();
      return true;
    }

    if (evt.id == EVENT_INVITATION_ACCEPTED) {
      server.joinChannel((IrcChan)evt.arg);
      ((Frame)evt.target).dispose();
      return true;
    }

    if (evt.target == channellist) {
      if (channellist.getSelectedIndex() == -1) {
	channelviewbutton.disable();
	channeljoinbutton.disable();	
      }
      else {
	channelviewbutton.enable();
	channeljoinbutton.enable();	
      }
    }

    if ((evt.target == channellist) || (evt.target == userlist)) {
      if ((channellist.getSelectedIndex() != -1) 
	  && (userlist.getSelectedIndexes().length >0) 
	  && (server.isUserOnChannel(ego,channels[channellist.getSelectedIndex()]))) {
	channelinvitebutton.enable();
      }
      else {
	channelinvitebutton.disable();
      }
    }
    return super.handleEvent(evt);
  }

  /**
   * Show a string in the status field
   * @param s The string to show.
   */
  public void log(String s) {
    if( logTF != null )
      {
	logTF.setText( s );
	logTF.select( 0, 0 );
      }
  }

  /**
   * Show a string in the status field. Why do we have two of these?
   * @param s The string to show.
   */
  public void showStatus( String s ) {
    log( s );
    // applet.showStatus( s );
  }

  /**
   * Update the user list when a new user joins IRC
   * @param nick The nickname of the new user
   */
  public synchronized void addUser(String nick) {
    if (userlist != null) updateuserlist();
  }

  /**
   * Update the user list when a user leaves.
   * @param nick The nickname of the user that left.
   */
  public synchronized void userLeft(String nick) {
    updateuserlist();
  }

  LoginDialog loginDialog;

  /**
   * Displays a login dialog box
   */
  public void noConnection() {
    // showLogin();
    Component comp;
    for (comp = getParent(); !(comp instanceof Frame); comp = comp.getParent());
		
    if (loginDialog != null)
      loginDialog.dispose();
			
    loginDialog = new LoginDialog((Frame)comp, "Login to Virtual School", this, map);
    loginDialog.show();
  }

  /**
   * Return the applet associated with the world client.
   */
  public Applet getApplet() {
    return this;
  }

  /**
   * Handle when someone leaves a channel. Does nothing now.
   * @param chan Channel that was left.
   */
  public void leftChannel(IrcChan chan) {
    return;
  }
	
  /**
   * Log a user into the system.
   * @param ego The object associated with the user.
   */
  public void loginUser(User ego) {
    if (debug)
      System.out.println("logging in");
    this.ego = ego;
    try {
      String proxyhost = null;
      if (getDocumentBase().getHost().equals("")) {
	proxyhost = InetAddress.getLocalHost().getHostName();
      } else {
	proxyhost = getDocumentBase().getHost();
      }
      server = new ServerIRC(ego,proxyhost,ircport,this);
      System.out.println("logged in");
    } catch( IOException e ) {
      new SmartFrame( "Couldn't connect to server, try again later" );
      noConnection();
    }
    showLists();
  }

  /**
   * joinSession() joins or creates a session
   */
  public void joinSession(String sessionName){
    if (debug)
      System.out.println("WorldClient: joinSession: "+sessionName);
		
    // the third parameter in IrcChan is supposed to be the topic
    // we should get the current topic of that channel and reuse it.
    IrcChan ch = new IrcChan(WORLD_NAME, sessionName, "TopicPlaceHolder - get from channel");
    server.joinChannel(ch);
  }

  /**
   * startCommunication() starts a new commlet and invites the users[]
   * 
   */
  public void startCommunication(String commletname, User users[], String topic ) {
    server.startCommunication(commletname, users, topic);
  }

  /**
   * Return all users using our Programm, i.e. all that are connected
   * to the IRCServer and who are in "pattern"
   */
  public  User[] getUsers( String pattern, boolean como_only ) {
    return server.getUsers(pattern, como_only);
  }


  /**
   * Return all the channels (matching pattern)
   */
  public  IrcChan[] getChannels( String pattern ){
    return server.getChannels(pattern, false);
  }


}
