// MultiColumn
// A List box that supports multiple columns.

/*
 * $Header: /home/nie/cvsroot/prototype/vt/cs/widgets/MultiColumn.java,v 1.1.1.1 1996/09/17 05:23:14 isenhour Exp $
 */


package vt.cs.widgets;

import java.awt.*;
import java.util.Vector;

public class MultiColumn extends BorderPanel
{
	private final boolean debug = false;
	
	MultiColumnCallback callback;	// what to call back to 
	String title[];			// column titles
	int cpos[];			// column x positions
	Vector list[];			// columns of the list
	Scrollbar sb;			// scrollbar at the right side
	int width, height;		// size, minus the scrollbar
	Insets in;			// used space around the border
	int sbwidth;			// width of the scrollbar
	int th;				// height of title bar
	Image bim;			// backing image
	Graphics bg;			// backing graphics
	FontMetrics fnm;		// drawing font size
	int coldrag = -1;		// column being resized
	int sel = -1;			// selected row
	int top = 0;			// first row displayed
	long last;			// last mouse click time
	int rows = 5;				// initial number of rows in list

	// Create a new list with the given column titles
	public MultiColumn(String t[], int rows)
	{
	super(3, new Color(50,50,50), new Color(220,220,220));
	if (debug)
		System.out.println("MultiColumn: Constructor: "+t[0]+" "+t[1]);
	title = new String[t.length];
	for(int i=0; i<t.length; i++)
		title[i] = t[i];
	list = new Vector[t.length];
	for(int i=0; i<t.length; i++)
		list[i] = new Vector();
	setLayout(null);
	sb = new Scrollbar(Scrollbar.VERTICAL, 0, 1, 0, 0);
	add(sb);
	if (debug)
		System.out.println("MultiColumn: Constructor: end");
	}

	// Create a new list that calls back to the given object on
	// single or double clicks.
	public MultiColumn(String t[], int rows, MultiColumnCallback c)
	{
	this(t, rows);
	if (debug)
		System.out.println("MultiColumn: Constructor: "+t[0]+" "+t[1]);
	callback = c;
	if (debug)
		System.out.println("MultiColumn: Constructor: end");
	}

	// addItem
	// Add a row to the list
	public void addItem(String item[])
	{
	if (debug)
		System.out.println("MultiColumn: addItem()");
	for(int i=0; i<title.length; i++)
		list[i].addElement(item[i]);
	repaint();
	compscroll();
	}

	// addItems
	// Add several rows to the list
	public void addItems(String item[][])
	{
	if (debug)
		System.out.println("MultiColumn: addItems()");
	for(int i=0; i<item.length; i++)
		for(int j=0; j<title.length; j++)
			list[j].addElement(item[i][j]);
	repaint();
	compscroll();
	}

	// getItem
	// Returns the contents of a given row
	public String []getItem(int n)
	{
	if (debug)
		System.out.println("MultiColumn: getItem()");
	String r[] = new String[title.length];
	for(int i=0; i<title.length; i++)
		r[i] = (String)list[i].elementAt(n);
	return r;
	}

	// selected
	// Return the selected row
	public int selected()
	{
	if (debug)
		System.out.println("MultiColumn: selected()");
	return sel;
	}
	
	public void select(int index) {
		if (debug)
			System.out.println("MultiColumn: select()");
		sel = index;
		repaint();
	}

	// deleteItem
	// Remove one row from the list
	public void deleteItem(int n)
	{
	if (debug)
		System.out.println("MultiColumn: deleteItem()");
	for(int i=0; i<title.length; i++)
		list[i].removeElementAt(n);
	if (n == sel) sel = -1;
	repaint();
	compscroll();
	}

	// clear
	// Remove everything from the list
	public void clear()
	{
	if (debug)
		System.out.println("MultiColumn: clear()");
	for(int i=0; i<title.length; i++)
		list[i].removeAllElements();
	sel = -1;
	repaint();
	sb.setValues(0, 1, 0, 0);
	}

	// reshape
	// Called when this component gets resized
	public void reshape(int nx, int ny, int nw, int nh)
	{
	if (debug)
		System.out.println("MultiColumn: reshape()");
	if (nw != width+sbwidth || nh != height) {
		in = insets();
		sbwidth = sb.minimumSize().width;
		width = nw-sbwidth - (in.left + in.right);
		height = nh - (in.top + in.bottom);
		sb.reshape(width+in.left, in.top, sbwidth, height);

		// Space columns evenly
		cpos = new int[title.length+1];
		for(int i=0; i<=title.length; i++)
			cpos[i] = width*i / title.length;

		// Force creation of a new backing image and re-painting
		bim = null;
		repaint();
		compscroll();
		}
	super.reshape(nx, ny, nw, nh);
	}

	private void setBim() {
		bim = createImage(width, height);
		bg = bim.getGraphics();
//		bg.setFont(new Font("timesRoman", Font.PLAIN, 12));
		fnm = bg.getFontMetrics();
		th = fnm.getHeight() + 4;
	}
	
	// paint
	// Blit the backing image to the front
	public void paint(Graphics g)
	{
	if (debug)
		System.out.println("MultiColumn: paint: Size= "+size());
	super.paint(g);
	if (bim == null) {
		// This is the first rendering
		setBim();
		render();
		compscroll();
		}
	g.drawImage(bim, in.left, in.top, this);
	}

	// update
	// Called sometime after repaint()
	public void update(Graphics g)
	{
	if (debug)
		System.out.println("MultiColumn: update()");
	if (fnm != null) {
		render();
		paint(g);
		}
	}

	// render
	// Re-draw the list into the backing image
	public void render()
	{
	if (debug)
		System.out.println("MultiColumn: render()");
		
	if (bim == null) {
		// This is the first rendering
		setBim();
	}
		
	int fh = fnm.getHeight(),	// useful font metrics
	    fd = fnm.getDescent(),
	    fa = fnm.getAscent();
	int bot = Math.min(top+rows()-1, list[0].size()-1);

	// Clear title section and list
	bg.setColor(Color.lightGray);
	bg.fillRect(0, 0, width, th);
	bg.setColor(Color.white);
	bg.fillRect(0, th, width, height-th);

	// Mark the selected row
	if (sel >= top && sel <= bot) {
		bg.setColor(Color.lightGray);
		bg.fillRect(0, th+(sel-top)*fh, width, fh);
		}

	// Draw each column
	for(int i=0; i<title.length; i++) {
		int x = cpos[i], w = cpos[i+1]-x-1;

		// Column title
		bg.setColor(Color.white);
		bg.drawLine(x, 0, x+w, 0);
		bg.drawLine(x, 1, x+w-1, 1);
		bg.drawLine(x, 0, x, th-1);
		bg.drawLine(x+1, 0, x+1, th-2);
		bg.setColor(Color.black);
		bg.drawLine(x, th-1, x+w, th-1);
		bg.drawLine(x, th-2, x+w-1, th-2);
		bg.drawLine(x+w, th-1, x+w, 0);
		bg.drawLine(x+w-1, th-1, x+w-1, 1);
		int tw = fnm.stringWidth(title[i]);
		if (tw < w-6)
			bg.drawString(title[i], x+(w-tw)/2, th-fd-2);

		// Column items
		bg.setColor(Color.lightGray);
		bg.drawLine(x+w-1, th, x+w-1, height);
		bg.setColor(Color.black);
		bg.drawLine(x+w, th, x+w, height);
		for(int j=top; j<=bot; j++) {
			String s = (String)list[i].elementAt(j);
			while(fnm.stringWidth(s) > w-3)
				s = s.substring(0, s.length()-1);
			bg.drawString(s, x+1, th+(j+1-top)*fh-fd);
			}
		}
	}

	// mouseDown
	// Select a list item or a column to drag
	public boolean mouseDown(Event e, int x, int y)
	{
	if (debug)
		System.out.println("MultiColumn: mouseDown()");
	x -= in.left;
	y -= in.top;
	coldrag = -1;
	if (y < th) {
		// Click in title bar
		for(int i=1; i<title.length; i++)
			if (Math.abs(cpos[i] - x) < 3) {
				// clicked on a column separator
				coldrag = i;
				}
		}
	else {
		// Item chosen from list
		int row = (y-th)/fnm.getHeight() + top;
		if (row < list[0].size()) {
			// Double-click?
			boolean dclick = false;
			if (e.when-last < 1000 && sel == row)
				dclick = true;
			else
				last = e.when;

			sel = row;
			repaint();
			if (callback != null) {
				// Callback the right function
				if (dclick) callback.doubleClick(this, row);
				else	    callback.singleClick(this, row);
				}
			else {
				// Send an event
				getParent().postEvent(
					new Event(this,
						  Event.ACTION_EVENT,
						  dclick?"Double":"Single"));
				}
			}
		}
	return true;
	}

	// mouseDrag
	// If a column is selected, change it's width
	public boolean mouseDrag(Event e, int x, int y)
	{
	if (debug)
		System.out.println("MultiColumn: mouseDrag()");
	x -= in.left;
	y -= in.top;
	if (coldrag != -1) {
		if (x > cpos[coldrag-1]+3 && x < cpos[coldrag+1]-3) {
			cpos[coldrag] = x;
			repaint();
			}
		}
	return true;
	}

	// handleEvent
	// Deal with scrollbar movement
	public boolean handleEvent(Event evt)
	{
	if (debug)
		System.out.println("MultiColumn: handleEvent()");
	if (evt.target == sb) {
		// scrollbar moved
		top = sb.getValue();
		compscroll();
		repaint();
		}
	return super.handleEvent(evt);
	}

	// compscroll
	// Re-compute the size of the scrollbar
	private void compscroll()
	{
	if (debug)
		System.out.println("MultiColumn: compscroll()");
	if (fnm == null)
		return;		// not visible
	int r = rows();
	int c = list[0].size() - r;
	sb.setValues(top, r==0?1:r, 0, c<0?0:c);
	}

	// rows
	// Returns the number of rows visible in the list
	private int rows()
	{
	if (debug)
		System.out.println("MultiColumn: rows()");
	return Math.min(height / fnm.getHeight() - 1, list[0].size());
	}
	
	public int getRows() {
	if (debug)
		System.out.println("MultiColumn: getRows()");
		return list[0].size();
	}
	
	public Dimension preferredSize() {
		if (debug)
			System.out.println("MultiColumn: preferredSize()");
		return minimumSize();
	}

	
	public Dimension minimumSize() {
		if (debug)
			System.out.println("MultiColumn: minimumSize()");
		if (fnm == null) {
			Graphics tg = getGraphics();
			fnm = tg.getFontMetrics();
		}
		int rowPixelHeight = fnm.getHeight() + 4;
		int minWidth = 0;
		for(int i=0; i<title.length; i++)
			minWidth += fnm.stringWidth(title[i]);
		return new Dimension(150, rows*rowPixelHeight);
	}

	
}


