/*
 * %W% %E%  James Bo Begole
 *
 * Copyright (c) 1997 James Begole,  All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * James Begole MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. James Begole 
 * SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */
package vt.cs.widgets;

import java.awt.*;
import java.awt.image.*;
import java.util.*;

/*
 * The radar view component of a RadarPane
 *
 * @version 	$Revision: 1.12 $
 * @author      James Begole
 */


/****************************************
RadarView is the component that displays the radar view.
****************************************/

public class RadarView extends Canvas implements java.util.Observer {
  public static final int defaultScaleFactor = 6;
  public int outlineWidth = 2;
  public int crossHairSize = 2;


  private Image bgImage = null;
  private Image scaledBgImage = null;
  private Image oldScaledBgImage = null;
  private RadarPaneClientPanel clientPanel = null;
  private RadarPane scroller = null;
  public Hashtable participants = new Hashtable();
  private boolean localUserSet = false;
  private boolean drawCursors = false;
  private boolean drawLabels = false;

  private MURadarPaneParent muparent = null;

  private Scalable scaler = null;

  public RadarView(RadarPaneClientPanel clientPanel, RadarPane scroller) {
    this.clientPanel = clientPanel;
    this.scroller = scroller;
  }

  boolean clientChanged = true;
  public void setClientChanged(boolean value) {
    clientChanged = value;
    if (value) {
      setLocalUser();
      repaint();
    }
  }

  private boolean radarPaneChange = false;
  public void setRadarPaneChange(boolean value) {
    radarPaneChange = value;
    if (value) {
      setLocalUser();
      repaint();
    }
  }

  private boolean visible = false;
  public void setVisible(boolean value) {
    super.setVisible(value);
    visible = value;
  }

  public Scalable getScaler() {
	return scaler;
  }
  public void setScaler(Scalable scaler) {
	this.scaler = scaler;
  }

  public void setDrawCursors(boolean value) {
    drawCursors = value;
  }
  public void setDrawLabels(boolean value) {
    drawLabels = value;
  }


  public void update(Graphics g) {
    debug("update");
    paint(g);
  }
  
  Dimension clientSize = null;
  Dimension size = null;
  int imgX, imgY, imgWidth, imgHeight;
  Dimension imgSize = null;
  //  Insets insets = null;
  Graphics scaledBgGraphics = null;

  boolean scaledImageReady = false;
  ScaleThread scaleThread = null;

  void setScaledImageReady(boolean value, Image image) {
    scaledImageReady = value;
    //    scaledBgImage = image;
    //    scaleThread.stop();
    scaleThread = null;
    repaint();
  }
    
  Image offscreen;

  /**
   * null out the offscreen buffer as part of invalidation
   */
  public void invalidate() {
      super.invalidate();
      offscreen = null;
  }


  public void paint(Graphics g) {
    debug("paint()");
    if (!visible) {
      debug("not visible");
      return;
    } else debug("is visible");
   
    if (!clientPanel.getComponent(0).isValid()) {
      debug("child is not valid");
      clientPanel.getComponent(0).validate();
      setClientChanged(true);
      //   return;
    }

    if(offscreen == null) {
      offscreen = createImage(getSize().width, getSize().height);
    }

    Graphics og = offscreen.getGraphics();
    og.setClip(0,0,getSize().width, getSize().height);
    og.setColor(clientPanel.getComponent(0).getBackground());
    og.fillRect(0,0, getSize().width, getSize().height);
         
    Dimension checkClientSize = clientPanel.getComponent(0).getSize();
    if (checkClientSize.equals(new Dimension(0, 0))) {
      debug("child size = 0,0");
      return;
    }
     
    Scalable s = scaler;
    if ((s == null) && (clientPanel.getComponent(0) instanceof Scalable))
	s = (Scalable)clientPanel.getComponent(0);

    //if (clientPanel.getComponent(0) instanceof Scalable) {
    if (s != null) {
      debug("calling Scalable");
      // if this object knows how to draw itself scaled,
      //if (((Scalable)clientPanel.getComponent(0)).scaleSelf()){
      if (s.scaleSelf()) {
	// and wants to, let it
	s.paintScaled(og, getSize());
	
	drawUserRects(og);
	
	if (offscreen != null)
	  g.drawImage(offscreen, 0, 0, null);
	og.dispose();
	clientChanged = false;
	clientSize = clientPanel.getComponent(0).getSize();
	imgSize = new Dimension(getSize().width, getSize().height);
	return;
      }
    } else {
      debug("client comp = "+clientPanel.getComponent(0));
    }
    
   
    //Otherwise, do the fancy image capture and scaling
 
    if ((bgImage == null) || 
	clientChanged ||
	(clientSize == null) ||
	(!(clientSize.equals(checkClientSize )))) {
      debug("New bgImage");
      clientChanged = false;
      // Client has changed size, get a new bgImage
      clientSize = checkClientSize;

      bgImage = createImage(clientSize.width, clientSize.height);
      Graphics offgraphics = bgImage.getGraphics();
      //Clear the BgImage to have the client's background color
      //Shouldn't have to do this - Windows AWT bug
      offgraphics.setColor(clientPanel.getComponent(0).getBackground());
      offgraphics.fillRect(0,0, clientSize.width, clientSize.height);
      clientPanel.getComponent(0).printAll(offgraphics);
      if (scaledImageReady)
	oldScaledBgImage = scaledBgImage;
      else
	oldScaledBgImage = null;
      scaledBgImage = null;
      if (oldScaledBgImage == null) {  // first time around only
	// make a blank one
	oldScaledBgImage = createImage(getSize().width, getSize().height);
	Graphics tempGraphics = oldScaledBgImage.getGraphics();
	tempGraphics.setColor(Color.lightGray);
	tempGraphics.fillRect(0,0,getSize().width,getSize().height);
      }
    }

    Dimension checkSize = getSize();
    if ((size == null) ||
    	(scaledBgImage == null) ||
	(!size.equals(checkSize))) {
      debug("New scaledBgImage");
      size = checkSize;
      scaledImageReady = false;

      imgX = 0;
      imgY = 0;
      imgWidth = size.width;
      imgHeight = size.height;
      imgSize = new Dimension(imgWidth, imgHeight);

      //      if (!scaledImageLoading) {
      //scaledBgImage =  createImage(imgWidth, imgHeight);
      //scaledBgGraphics = scaledBgImage.getGraphics();
      
      debug("pre  scaledBgGraphics.drawImage "+new java.util.Date());
      scaledBgImage = bgImage.getScaledInstance(imgWidth, imgHeight, Image.SCALE_FAST);
      
      debug("pre new ScaleThread"+new Date());
      if (scaleThread != null) {
	debug("stopping old scaleThread");
	scaleThread.discard();
	scaleThread = null;
      }
      
       scaleThread = new ScaleThread(scaledBgImage, this);
 
				      
      debug("post  scaledBgGraphics.drawImage "+new java.util.Date());
      
      //scaledImageLoading = true;
      //}
    }
    
    if (!scaledImageReady) {
      debug("scaled Image not ready");
      if (oldScaledBgImage != null)  // get something on screen
	og.drawImage(oldScaledBgImage, 0, 0, this);
      else 
	og.clearRect(0, 0, getSize().width, getSize().height);
      
      og.setColor(Color.black);
      Font oldFont = og.getFont();
      Font font = new Font("Serif", Font.PLAIN, 20);
      og.setFont(font);
      String message = "Scaling new view...";
      FontMetrics fontMetrics = og.getFontMetrics();
      int xPos = (size.width - fontMetrics.stringWidth(message))/2;
      og.drawString(message, xPos, getSize().height/2);
      og.setFont(oldFont);
    } else {
      oldScaledBgImage = null;
      debug("pre  og.drawImage "+new java.util.Date());
      og.drawImage(scaledBgImage, 0, 0, null);
      debug("post  og.drawImage "+new java.util.Date());
    }


    if (radarPaneChange) {
      radarPaneChange = false;
      setLocalUser();
    }

    drawUserRects(og);

    if (offscreen != null)
      g.drawImage(offscreen, 0, 0, null);
    og.dispose();
    debug("end paint()");
  } 

  public void setMURadarPaneParent(MURadarPaneParent muparent) {
    this.muparent = muparent;
  }

  private Object localUserKey = new Object();
  private Color localUserColor = Color.black;

  public void setLocalUserColor(Color color) {
    localUserColor = color;
    repaint();
  }

  public void setLocalUser() {
    debug("clientPanel"+clientPanel);
    if ((clientPanel == null)) {
      clearLocalUser();
      return;
    }

    Component comps[] = clientPanel.getComponents();
    Component comp = null;
    if (comps != null) {
      comp = clientPanel.getComponent(0);
      debug("comp"+comp);
      if (comp == null) {
	clearLocalUser();
	return;
      }
    }

    Point clientLoc = comp.getLocation();
    Dimension visible = clientPanel.getSize();
   debug("sizes: "+clientLoc+" :"+visible+" :"+imgSize+" :"+clientSize);

   Rectangle toScale = new Rectangle(-clientLoc.x, -clientLoc.y,
				     visible.width,
				     visible.height);
   if ((muparent != null) &&
       (participants.get(localUserKey) != null)) {
     muparent.updateBounds(toScale);
     debug("called muparent.updateBounds(toScale)"+toScale);
   } else {
     addLocalViewPort( new ColorRectangle(toScale, 
					  localUserColor));
   }
   repaint();
  }
  
  private Rectangle scaleRectangle(Rectangle toScale,
				   Dimension relativeSize,
				   Dimension fullSize) {
    debug("scaleRectangle "+toScale+fullSize+relativeSize);
    if ((toScale == null) || (relativeSize == null) || (fullSize == null))
      return null;

    float horizScale = (float)relativeSize.width/(float)fullSize.width;
    float vertScale = (float)relativeSize.height/(float)fullSize.height;
    int scaledX = Math.round(toScale.x * horizScale);
    int scaledWidth = Math.round(toScale.width * horizScale);
    if ((scaledWidth+scaledX)>(relativeSize.width-1)) 
      scaledWidth = relativeSize.width-1-scaledX;
    int scaledY = Math.round(toScale.y * vertScale);
    int scaledHeight =  Math.round(toScale.height * vertScale);
    if ((scaledHeight+scaledY)>(relativeSize.height-1))
      scaledHeight = relativeSize.height-1-scaledY;

    Rectangle scaled =  new Rectangle(scaledX, scaledY, 
				      scaledWidth, scaledHeight);
    debug("scaleRectangle "+scaled);
    return scaled;
  }

  private  Point scalePoint(Point toScale,
			    Dimension relativeSize,
			    Dimension fullSize) {
    debug("scalePoint "+toScale+fullSize+relativeSize);
    if ((toScale == null) || (relativeSize == null) || (fullSize == null))
      return null;

    float horizScale = (float)relativeSize.width/(float)fullSize.width;
    float vertScale = (float)relativeSize.height/(float)fullSize.height;
    int scaledX = Math.round(toScale.x * horizScale);
    int scaledY = Math.round(toScale.y * vertScale);
    return new Point(scaledX, scaledY);
 }

  private void clearLocalUser() {
    participants.remove(localUserKey);
  }

  public void addParticipant(Object key, RadarViewPort viewPort) {
    debug("addParticipant "+key+" : "+viewPort);
    participants.put(key, viewPort);
    if (viewPort instanceof Observable) {
	((Observable)viewPort).addObserver(this);
    }
    repaint();
  }
  public void addLocalViewPort(RadarViewPort viewPort){
    addParticipant(localUserKey, viewPort);
  }

  public void removeParticipant(Object key) {
    participants.remove(key);
    repaint();
  }
  
  
  private void drawUserRects(Graphics g) {
    debug("drawUserRects");

    if (!localUserSet) {
      setLocalUser();
      localUserSet = true;
    }

    //    g.setXORMode(Color.white);
    for(Enumeration e = participants.elements(); e.hasMoreElements(); ) {
      RadarViewPort user = (RadarViewPort)e.nextElement();
      Rectangle scaled = scaleRectangle(user.getBounds(), 
					getSize(), 
					clientPanel.getComponent(0).getSize());
      g.setColor(user.getColor()); 
      debug("userRect scaled = "+scaled);
      if (scaled != null)
	for(int i=0; i<outlineWidth; i++)
	  g.drawRect(scaled.x+i, scaled.y+i, 
		     scaled.width-2*i, scaled.height-2*i);
      if (drawCursors) {
	Point position = scalePoint(user.getCursorPosition(),
				    getSize(), 
				    clientPanel.getComponent(0).getSize());
	if (position != null) {
	  g.drawLine(position.x, position.y+crossHairSize, 
		     position.x, position.y-crossHairSize);
	  g.drawLine(position.x-crossHairSize, position.y, 
		     position.x+crossHairSize, position.y);
	}
      }
      if (drawLabels) {
	String label = user.getLabel();
	if ((label != null) && (scaled != null)) {
	  // keep the label within the user Rect
	  Shape oldClip = g.getClip();
	  g.setClip(scaled);
	  g.drawString(label, scaled.x+outlineWidth+5, 
		       scaled.y+scaled.height-(outlineWidth+5));
	  g.setClip(oldClip);
	}
      }
    }
    //    g.setPaintMode();
    debug("end drawUserRects");
  }

  public Dimension getMinimumSize() {
    Dimension dim = null;
    if ((clientPanel == null) || !clientPanel.isValid())
      dim = new Dimension(200, 200);
    else {
      Dimension tempSize = clientPanel.getComponent(0).getSize();
      dim = new Dimension(tempSize.width/defaultScaleFactor, 
			  tempSize.height/defaultScaleFactor);
    }
    debug("getMinimumSize() returning "+dim);
    return dim;
  }

  public Dimension getPreferredSize() {
    debug("getPreferredSize()");
    return getMinimumSize();
  }

  Point mouseDown = null;
  Point initialUserRectLoc = null;

  private void relativeMove(int x, int y) {
    Rectangle localUser = scaleRectangle(
	     ((RadarViewPort)participants.get(localUserKey)).getBounds(),
	     getSize(), 
	     clientPanel.getComponent(0).getSize());
    if (localUser != null)
      if (mouseDown == null) {  
	// Center the box on x,y
	moveBox(x-(localUser.width/2), y-(localUser.height/2));
      } else {  
	// move it relative to initial position
	moveBox(initialUserRectLoc.x+(x-mouseDown.x),
		initialUserRectLoc.y+(y-mouseDown.y));
      }
  }

  public boolean mouseDown(Event evt, int x, int y) {
    debug("mouseDown");
    if ((clientSize == null) || (imgSize == null))
      return false;

    if (evt.shiftDown()) {
      debug("shift Down");
      setClientChanged(true);
      return true;
    }

    Rectangle localUser = scaleRectangle(
	     ((RadarViewPort)participants.get(localUserKey)).getBounds(),
	     getSize(), 
	     clientPanel.getComponent(0).getSize());
    if (localUser != null)
      if (localUser.contains(x, y) && !evt.metaDown()) {  
	// user clicked inside box
	debug("inside box");
	mouseDown = new Point(x,y);
	initialUserRectLoc = new Point(localUser.x, localUser.y);
      }
    relativeMove(x,y);
    return true;
  }

  public boolean  mouseDrag(Event evt, int x, int y) {
    if ((clientSize == null) || (imgSize == null) || evt.shiftDown())
      return false;

    relativeMove(x,y);
    return true;
  }

  public boolean  mouseUp(Event evt, int x, int y) {
    if ((clientSize == null) || (imgSize == null) || evt.shiftDown()){
      mouseDown = null;
      initialUserRectLoc = null;
      return false;
    }

    relativeMove(x,y);
    mouseDown = null;
    initialUserRectLoc = null;
    return true;
  }

  private void moveBox(int x, int y) {  
    Rectangle localUser = scaleRectangle(
	     ((RadarViewPort)participants.get(localUserKey)).getBounds(),
	     getSize(), 
	     clientPanel.getComponent(0).getSize());

    // Make sure x is not off screen to the left
    x = Math.max(x, 0);
    // or to the right
    if (localUser != null)
      x = Math.min(x, getSize().width-localUser.width);
    debug("getSize().width = "+getSize().width+" localUser.width = "+
	  localUser.width+" x = "+x );
      
    // and make sure y is not off the top
    y = Math.max(y, 0);
    // or off the bottom
    if (localUser != null)
      y = Math.min(y, getSize().height-localUser.height);
      
    float horizScale = (float)clientSize.width/(float)imgSize.width;
    float vertScale = (float)clientSize.height/(float)imgSize.height;

    int scaledX = Math.round(x*horizScale);
    int scaledY = Math.round(y*vertScale);
    debug("scaledX = "+scaledX+", scaledY = "+scaledY);
    scroller.setScrollPosition(scaledX, scaledY);
    repaint();
  }

// --------- Observer Methods ---------------
  public void update(java.util.Observable o,
                              Object arg){
    repaint();
  }
// --------- end Observer Methods ---------------


  /*
    // debug overrides
    public void validate() {
    debug("validate()");
    super.validate();
    } 

    public void invalidate() {
    debug("invalidate()");
    super.invalidate();
    //   if (getParent() != null)
    //     getParent().doLayout();
      
    }
    */

  private static final boolean debug = false;
  private void debug(String message){
    if (debug)
      System.err.println(getClass().getName()+": "+message);
  }
}


class ScaleThread extends Thread implements ImageObserver {
  Image image = null;
  RadarView radarView = null;

  ScaleThread(Image image, RadarView radarView) {
    this.image = image;
    this.radarView = radarView;
    try {
      this.setPriority(getPriority()-1);
    } catch (SecurityException sec) {
      // do nothing
    }
    start();
  }

  public void run() {

    Toolkit toolkit = Toolkit.getDefaultToolkit();
    debug("pre prepareImage "+new Date());
    toolkit.prepareImage(image, -1, -1, this);
    debug("post prepareImage "+new Date());
    /*     
	   ImageProducer prod = image.getSource();
	   debug("startProduction"+new Date());
	   prod.startProduction(this);
	   debug("startProduction"+new Date());
	   */

  }

  private boolean discard = false;
  public void discard() {
    discard = true;
  }

  public boolean imageUpdate(Image img,
			     int infoflags,
			     int x,
			     int y,
			     int width,
			     int height) {
    /*  debug("img = "+img +
	", infoflags = " +infoflags+
	", x = "+ x+
	", y = "+y+
	", width =" +width+
	", height= "+height); */
	
    if (discard)
      return false;

    if (((infoflags & FRAMEBITS) != 0) ||
	((infoflags & ALLBITS) != 0)) {
      debug("Image complete.  infoflags="+infoflags);
      radarView.setScaledImageReady(true, image);
    }
    return true;
  }

  private static final boolean debug = false;
  private void debug(String message){
    if (debug)
      System.err.println(getClass().getName()+": "+message);
  }
}



